# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/airt-client',
                'doc_host': 'https://airtai.github.io',
                'git_url': 'https://github.com/airtai/airt-client',
                'lib_path': 'airt'},
  'syms': { 'airt.cli.api_key': { 'airt.cli.api_key.create': ('cli_key.html#create', 'airt/cli/api_key.py'),
                                  'airt.cli.api_key.details': ('cli_key.html#details', 'airt/cli/api_key.py'),
                                  'airt.cli.api_key.ls': ('cli_key.html#ls', 'airt/cli/api_key.py'),
                                  'airt.cli.api_key.revoke': ('cli_key.html#revoke', 'airt/cli/api_key.py')},
            'airt.cli.cli': {},
            'airt.cli.db': { 'airt.cli.db.details': ('cli_datablob.html#details', 'airt/cli/db.py'),
                             'airt.cli.db.from_azure_blob_storage': ('cli_datablob.html#from_azure_blob_storage', 'airt/cli/db.py'),
                             'airt.cli.db.from_clickhouse': ('cli_datablob.html#from_clickhouse', 'airt/cli/db.py'),
                             'airt.cli.db.from_local': ('cli_datablob.html#from_local', 'airt/cli/db.py'),
                             'airt.cli.db.from_mysql': ('cli_datablob.html#from_mysql', 'airt/cli/db.py'),
                             'airt.cli.db.from_s3': ('cli_datablob.html#from_s3', 'airt/cli/db.py'),
                             'airt.cli.db.ls': ('cli_datablob.html#ls', 'airt/cli/db.py'),
                             'airt.cli.db.rm': ('cli_datablob.html#rm', 'airt/cli/db.py'),
                             'airt.cli.db.tag': ('cli_datablob.html#tag', 'airt/cli/db.py'),
                             'airt.cli.db.to_datasource': ('cli_datablob.html#to_datasource', 'airt/cli/db.py')},
            'airt.cli.ds': { 'airt.cli.ds.details': ('cli_datasource.html#details', 'airt/cli/ds.py'),
                             'airt.cli.ds.dtypes': ('cli_datasource.html#dtypes', 'airt/cli/ds.py'),
                             'airt.cli.ds.head': ('cli_datasource.html#head', 'airt/cli/ds.py'),
                             'airt.cli.ds.ls': ('cli_datasource.html#ls', 'airt/cli/ds.py'),
                             'airt.cli.ds.rm': ('cli_datasource.html#rm', 'airt/cli/ds.py'),
                             'airt.cli.ds.tag': ('cli_datasource.html#tag', 'airt/cli/ds.py'),
                             'airt.cli.ds.train': ('cli_datasource.html#train', 'airt/cli/ds.py')},
            'airt.cli.helper': { 'airt.cli.helper.authenicate_user': ('cli_helper.html#authenicate_user', 'airt/cli/helper.py'),
                                 'airt.cli.helper.customize_output_format': ( 'cli_helper.html#customize_output_format',
                                                                              'airt/cli/helper.py'),
                                 'airt.cli.helper.display_formated_table': ('cli_helper.html#display_formated_table', 'airt/cli/helper.py'),
                                 'airt.cli.helper.echo_formatted_output': ('cli_helper.html#echo_formatted_output', 'airt/cli/helper.py'),
                                 'airt.cli.helper.get_example_for_type': ('cli_helper.html#get_example_for_type', 'airt/cli/helper.py'),
                                 'airt.cli.helper.get_example_output_format': ( 'cli_helper.html#get_example_output_format',
                                                                                'airt/cli/helper.py'),
                                 'airt.cli.helper.get_phone_registration_status': ( 'cli_helper.html#get_phone_registration_status',
                                                                                    'airt/cli/helper.py'),
                                 'airt.cli.helper.humanize_date': ('cli_helper.html#humanize_date', 'airt/cli/helper.py'),
                                 'airt.cli.helper.humanize_number': ('cli_helper.html#humanize_number', 'airt/cli/helper.py'),
                                 'airt.cli.helper.humanize_size': ('cli_helper.html#humanize_size', 'airt/cli/helper.py'),
                                 'airt.cli.helper.requires_auth_token': ('cli_helper.html#requires_auth_token', 'airt/cli/helper.py'),
                                 'airt.cli.helper.requires_totp': ('cli_helper.html#requires_totp', 'airt/cli/helper.py'),
                                 'airt.cli.helper.requires_totp_or_otp': ('cli_helper.html#requires_totp_or_otp', 'airt/cli/helper.py'),
                                 'airt.cli.helper.separate_integers_and_strings': ( 'cli_helper.html#separate_integers_and_strings',
                                                                                    'airt/cli/helper.py')},
            'airt.cli.model': { 'airt.cli.model.details': ('cli_model.html#details', 'airt/cli/model.py'),
                                'airt.cli.model.evaluate': ('cli_model.html#evaluate', 'airt/cli/model.py'),
                                'airt.cli.model.ls': ('cli_model.html#ls', 'airt/cli/model.py'),
                                'airt.cli.model.predict': ('cli_model.html#predict', 'airt/cli/model.py'),
                                'airt.cli.model.rm': ('cli_model.html#rm', 'airt/cli/model.py')},
            'airt.cli.pred': { 'airt.cli.pred.details': ('cli_pred.html#details', 'airt/cli/pred.py'),
                               'airt.cli.pred.ls': ('cli_pred.html#ls', 'airt/cli/pred.py'),
                               'airt.cli.pred.rm': ('cli_pred.html#rm', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_azure_blob_storage': ('cli_pred.html#to_azure_blob_storage', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_clickhouse': ('cli_pred.html#to_clickhouse', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_local': ('cli_pred.html#to_local', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_mysql': ('cli_pred.html#to_mysql', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_pandas': ('cli_pred.html#to_pandas', 'airt/cli/pred.py'),
                               'airt.cli.pred.to_s3': ('cli_pred.html#to_s3', 'airt/cli/pred.py')},
            'airt.cli.token': {'airt.cli.token.token': ('cli_token.html#token', 'airt/cli/token.py')},
            'airt.cli.user': { 'airt.cli.user.create': ('cli_user.html#create', 'airt/cli/user.py'),
                               'airt.cli.user.details': ('cli_user.html#details', 'airt/cli/user.py'),
                               'airt.cli.user.disable': ('cli_user.html#disable', 'airt/cli/user.py'),
                               'airt.cli.user.enable': ('cli_user.html#enable', 'airt/cli/user.py'),
                               'airt.cli.user.ls': ('cli_user.html#ls', 'airt/cli/user.py'),
                               'airt.cli.user.register_phone_number': ('cli_user.html#register_phone_number', 'airt/cli/user.py'),
                               'airt.cli.user.reset_password': ('cli_user.html#reset_password', 'airt/cli/user.py'),
                               'airt.cli.user.update': ('cli_user.html#update', 'airt/cli/user.py')},
            'airt.cli.version': {'airt.cli.version.version': ('cli_version.html#version', 'airt/cli/version.py')},
            'airt.client': {},
            'airt.components.api_key': { 'airt.components.api_key.APIKey': ('api_keys.html#apikey', 'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.__init__': ( 'api_keys.html#apikey.__init__',
                                                                                      'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey._get_key_names': ( 'api_keys.html#apikey._get_key_names',
                                                                                            'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.as_df': ( 'api_keys.html#apikey.as_df',
                                                                                   'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.create': ( 'api_keys.html#apikey.create',
                                                                                    'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.details': ( 'api_keys.html#apikey.details',
                                                                                     'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.ls': ('api_keys.html#apikey.ls', 'airt/components/api_key.py'),
                                         'airt.components.api_key.APIKey.revoke': ( 'api_keys.html#apikey.revoke',
                                                                                    'airt/components/api_key.py')},
            'airt.components.client': { 'airt.components.client.Client': ('api_client.html#client', 'airt/components/client.py'),
                                        'airt.components.client.Client.__init__': ( 'api_client.html#client.__init__',
                                                                                    'airt/components/client.py'),
                                        'airt.components.client.Client._delete_data': ( 'api_client.html#client._delete_data',
                                                                                        'airt/components/client.py'),
                                        'airt.components.client.Client._get_data': ( 'api_client.html#client._get_data',
                                                                                     'airt/components/client.py'),
                                        'airt.components.client.Client._get_server_url_and_token': ( 'api_client.html#client._get_server_url_and_token',
                                                                                                     'airt/components/client.py'),
                                        'airt.components.client.Client._post_data': ( 'api_client.html#client._post_data',
                                                                                      'airt/components/client.py'),
                                        'airt.components.client.Client.get_token': ( 'api_client.html#client.get_token',
                                                                                     'airt/components/client.py'),
                                        'airt.components.client.Client.set_sso_token': ( 'api_client.html#client.set_sso_token',
                                                                                         'airt/components/client.py'),
                                        'airt.components.client.Client.set_token': ( 'api_client.html#client.set_token',
                                                                                     'airt/components/client.py'),
                                        'airt.components.client.Client.version': ( 'api_client.html#client.version',
                                                                                   'airt/components/client.py'),
                                        'airt.components.client._get_credentials': ( 'api_client.html#_get_credentials',
                                                                                     'airt/components/client.py')},
            'airt.components.datablob': { 'airt.components.datablob.DataBlob': ( 'api_datablob.html#datablob',
                                                                                 'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.__init__': ( 'api_datablob.html#datablob.__init__',
                                                                                          'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob._get_cloud_provider_and_region': ( 'api_datablob.html#datablob._get_cloud_provider_and_region',
                                                                                                                'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob._get_default_provider_and_regions': ( 'api_datablob.html#datablob._get_default_provider_and_regions',
                                                                                                                   'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob._get_tag_name_and_datasource_id': ( 'api_datablob.html#datablob._get_tag_name_and_datasource_id',
                                                                                                                 'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob._upload_to_s3_with_retry': ( 'api_datablob.html#datablob._upload_to_s3_with_retry',
                                                                                                          'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.as_df': ( 'api_datablob.html#datablob.as_df',
                                                                                       'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.delete': ( 'api_datablob.html#datablob.delete',
                                                                                        'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.details': ( 'api_datablob.html#datablob.details',
                                                                                         'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_azure_blob_storage': ( 'api_datablob.html#datablob.from_azure_blob_storage',
                                                                                                         'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_clickhouse': ( 'api_datablob.html#datablob.from_clickhouse',
                                                                                                 'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_local': ( 'api_datablob.html#datablob.from_local',
                                                                                            'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_mysql': ( 'api_datablob.html#datablob.from_mysql',
                                                                                            'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.from_s3': ( 'api_datablob.html#datablob.from_s3',
                                                                                         'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.is_ready': ( 'api_datablob.html#datablob.is_ready',
                                                                                          'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.ls': ( 'api_datablob.html#datablob.ls',
                                                                                    'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.progress_bar': ( 'api_datablob.html#datablob.progress_bar',
                                                                                              'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.set_default_cloud_provider': ( 'api_datablob.html#datablob.set_default_cloud_provider',
                                                                                                            'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.tag': ( 'api_datablob.html#datablob.tag',
                                                                                     'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.to_datasource': ( 'api_datablob.html#datablob.to_datasource',
                                                                                               'airt/components/datablob.py'),
                                          'airt.components.datablob.DataBlob.wait': ( 'api_datablob.html#datablob.wait',
                                                                                      'airt/components/datablob.py'),
                                          'airt.components.datablob._docstring_example': ( 'api_datablob.html#_docstring_example',
                                                                                           'airt/components/datablob.py')},
            'airt.components.datasource': { 'airt.components.datasource.DataSource': ( 'api_datasource.html#datasource',
                                                                                       'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.__init__': ( 'api_datasource.html#datasource.__init__',
                                                                                                'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.as_df': ( 'api_datasource.html#datasource.as_df',
                                                                                             'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.delete': ( 'api_datasource.html#datasource.delete',
                                                                                              'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.details': ( 'api_datasource.html#datasource.details',
                                                                                               'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.dtypes': ( 'api_datasource.html#datasource.dtypes',
                                                                                              'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.head': ( 'api_datasource.html#datasource.head',
                                                                                            'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.is_ready': ( 'api_datasource.html#datasource.is_ready',
                                                                                                'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.ls': ( 'api_datasource.html#datasource.ls',
                                                                                          'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.progress_bar': ( 'api_datasource.html#datasource.progress_bar',
                                                                                                    'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.tag': ( 'api_datasource.html#datasource.tag',
                                                                                           'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.train': ( 'api_datasource.html#datasource.train',
                                                                                             'airt/components/datasource.py'),
                                            'airt.components.datasource.DataSource.wait': ( 'api_datasource.html#datasource.wait',
                                                                                            'airt/components/datasource.py'),
                                            'airt.components.datasource._docstring_example': ( 'api_datasource.html#_docstring_example',
                                                                                               'airt/components/datasource.py')},
            'airt.components.model': { 'airt.components.model.Model': ('api_model.html#model', 'airt/components/model.py'),
                                       'airt.components.model.Model.__init__': ( 'api_model.html#model.__init__',
                                                                                 'airt/components/model.py'),
                                       'airt.components.model.Model.as_df': ('api_model.html#model.as_df', 'airt/components/model.py'),
                                       'airt.components.model.Model.delete': ('api_model.html#model.delete', 'airt/components/model.py'),
                                       'airt.components.model.Model.details': ('api_model.html#model.details', 'airt/components/model.py'),
                                       'airt.components.model.Model.evaluate': ( 'api_model.html#model.evaluate',
                                                                                 'airt/components/model.py'),
                                       'airt.components.model.Model.ls': ('api_model.html#model.ls', 'airt/components/model.py'),
                                       'airt.components.model.Model.predict': ('api_model.html#model.predict', 'airt/components/model.py'),
                                       'airt.components.model._docstring_example': ( 'api_model.html#_docstring_example',
                                                                                     'airt/components/model.py')},
            'airt.components.prediction': { 'airt.components.prediction.Prediction': ( 'api_prediction.html#prediction',
                                                                                       'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.__init__': ( 'api_prediction.html#prediction.__init__',
                                                                                                'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction._download_prediction_file_to_local': ( 'api_prediction.html#prediction._download_prediction_file_to_local',
                                                                                                                          'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.as_df': ( 'api_prediction.html#prediction.as_df',
                                                                                             'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.delete': ( 'api_prediction.html#prediction.delete',
                                                                                              'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.details': ( 'api_prediction.html#prediction.details',
                                                                                               'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.ls': ( 'api_prediction.html#prediction.ls',
                                                                                          'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_azure_blob_storage': ( 'api_prediction.html#prediction.to_azure_blob_storage',
                                                                                                             'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_clickhouse': ( 'api_prediction.html#prediction.to_clickhouse',
                                                                                                     'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_local': ( 'api_prediction.html#prediction.to_local',
                                                                                                'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_mysql': ( 'api_prediction.html#prediction.to_mysql',
                                                                                                'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_pandas': ( 'api_prediction.html#prediction.to_pandas',
                                                                                                 'airt/components/prediction.py'),
                                            'airt.components.prediction.Prediction.to_s3': ( 'api_prediction.html#prediction.to_s3',
                                                                                             'airt/components/prediction.py'),
                                            'airt.components.prediction._docstring_example': ( 'api_prediction.html#_docstring_example',
                                                                                               'airt/components/prediction.py')},
            'airt.components.progress_status': { 'airt.components.progress_status.ProgressStatus': ( 'api_progressstatus.html#progressstatus',
                                                                                                     'airt/components/progress_status.py'),
                                                 'airt.components.progress_status.ProgressStatus.__init__': ( 'api_progressstatus.html#progressstatus.__init__',
                                                                                                              'airt/components/progress_status.py'),
                                                 'airt.components.progress_status.ProgressStatus.is_ready': ( 'api_progressstatus.html#progressstatus.is_ready',
                                                                                                              'airt/components/progress_status.py'),
                                                 'airt.components.progress_status.ProgressStatus.progress_bar': ( 'api_progressstatus.html#progressstatus.progress_bar',
                                                                                                                  'airt/components/progress_status.py'),
                                                 'airt.components.progress_status.ProgressStatus.wait': ( 'api_progressstatus.html#progressstatus.wait',
                                                                                                          'airt/components/progress_status.py')},
            'airt.components.user': { 'airt.components.user.User': ('api_user.html#user', 'airt/components/user.py'),
                                      'airt.components.user.User.__init__': ('api_user.html#user.__init__', 'airt/components/user.py'),
                                      'airt.components.user.User._get_mfa_provision_url': ( 'api_user.html#user._get_mfa_provision_url',
                                                                                            'airt/components/user.py'),
                                      'airt.components.user.User.activate_mfa': ( 'api_user.html#user.activate_mfa',
                                                                                  'airt/components/user.py'),
                                      'airt.components.user.User.as_df': ('api_user.html#user.as_df', 'airt/components/user.py'),
                                      'airt.components.user.User.create': ('api_user.html#user.create', 'airt/components/user.py'),
                                      'airt.components.user.User.details': ('api_user.html#user.details', 'airt/components/user.py'),
                                      'airt.components.user.User.disable': ('api_user.html#user.disable', 'airt/components/user.py'),
                                      'airt.components.user.User.disable_mfa': ( 'api_user.html#user.disable_mfa',
                                                                                 'airt/components/user.py'),
                                      'airt.components.user.User.disable_sso': ( 'api_user.html#user.disable_sso',
                                                                                 'airt/components/user.py'),
                                      'airt.components.user.User.enable': ('api_user.html#user.enable', 'airt/components/user.py'),
                                      'airt.components.user.User.enable_mfa': ('api_user.html#user.enable_mfa', 'airt/components/user.py'),
                                      'airt.components.user.User.enable_sso': ('api_user.html#user.enable_sso', 'airt/components/user.py'),
                                      'airt.components.user.User.ls': ('api_user.html#user.ls', 'airt/components/user.py'),
                                      'airt.components.user.User.register_phone_number': ( 'api_user.html#user.register_phone_number',
                                                                                           'airt/components/user.py'),
                                      'airt.components.user.User.reset_password': ( 'api_user.html#user.reset_password',
                                                                                    'airt/components/user.py'),
                                      'airt.components.user.User.send_sms_otp': ( 'api_user.html#user.send_sms_otp',
                                                                                  'airt/components/user.py'),
                                      'airt.components.user.User.update': ('api_user.html#user.update', 'airt/components/user.py'),
                                      'airt.components.user.User.validate_phone_number': ( 'api_user.html#user.validate_phone_number',
                                                                                           'airt/components/user.py')},
            'airt.constant': {},
            'airt.docstring.helpers': { 'airt.docstring.helpers._extract_examples_from_docstring': ( 'docstring_helpers.html#_extract_examples_from_docstring',
                                                                                                     'airt/docstring/helpers.py'),
                                        'airt.docstring.helpers._format_output': ( 'docstring_helpers.html#_format_output',
                                                                                   'airt/docstring/helpers.py'),
                                        'airt.docstring.helpers._get_keywords': ( 'docstring_helpers.html#_get_keywords',
                                                                                  'airt/docstring/helpers.py'),
                                        'airt.docstring.helpers._replace_keywords': ( 'docstring_helpers.html#_replace_keywords',
                                                                                      'airt/docstring/helpers.py'),
                                        'airt.docstring.helpers.run_examples_from_docstring': ( 'docstring_helpers.html#run_examples_from_docstring',
                                                                                                'airt/docstring/helpers.py')},
            'airt.helper': { 'airt.helper._get_json': ('api_helper.html#_get_json', 'airt/helper.py'),
                             'airt.helper.add_example_to_docs': ('api_helper.html#add_example_to_docs', 'airt/helper.py'),
                             'airt.helper.add_ready_column': ('api_helper.html#add_ready_column', 'airt/helper.py'),
                             'airt.helper.check_and_append_otp_query_param': ( 'api_helper.html#check_and_append_otp_query_param',
                                                                               'airt/helper.py'),
                             'airt.helper.delete_data': ('api_helper.html#delete_data', 'airt/helper.py'),
                             'airt.helper.dict_to_df': ('api_helper.html#dict_to_df', 'airt/helper.py'),
                             'airt.helper.ensure_is_instance': ('api_helper.html#ensure_is_instance', 'airt/helper.py'),
                             'airt.helper.generate_df': ('api_helper.html#generate_df', 'airt/helper.py'),
                             'airt.helper.get_attributes_from_instances': ( 'api_helper.html#get_attributes_from_instances',
                                                                            'airt/helper.py'),
                             'airt.helper.get_base_url': ('api_helper.html#get_base_url', 'airt/helper.py'),
                             'airt.helper.get_data': ('api_helper.html#get_data', 'airt/helper.py'),
                             'airt.helper.get_values_from_item': ('api_helper.html#get_values_from_item', 'airt/helper.py'),
                             'airt.helper.post_data': ('api_helper.html#post_data', 'airt/helper.py'),
                             'airt.helper.standardize_phone_number': ('api_helper.html#standardize_phone_number', 'airt/helper.py')},
            'airt.logger': { 'airt.logger.CLILoggerHandler': ('logger.html#cliloggerhandler', 'airt/logger.py'),
                             'airt.logger.CLILoggerHandler.emit': ('logger.html#cliloggerhandler.emit', 'airt/logger.py'),
                             'airt.logger.get_logger': ('logger.html#get_logger', 'airt/logger.py'),
                             'airt.logger.set_level': ('logger.html#set_level', 'airt/logger.py'),
                             'airt.logger.supress_timestamps': ('logger.html#supress_timestamps', 'airt/logger.py')},
            'airt.sanitizer': { 'airt.sanitizer.Logger._log': ('sanitize_secrets.html#logger._log', 'airt/sanitizer.py'),
                                'airt.sanitizer.new_publish_display_data': ( 'sanitize_secrets.html#new_publish_display_data',
                                                                             'airt/sanitizer.py'),
                                'airt.sanitizer.sanitize_secrets': ('sanitize_secrets.html#sanitize_secrets', 'airt/sanitizer.py'),
                                'airt.sanitizer.sanitized_print': ('sanitize_secrets.html#sanitized_print', 'airt/sanitizer.py')},
            'airt.testing.activate_by_import': { 'airt.testing.activate_by_import.activated': ( 'activatetestingenvironment.html#activated',
                                                                                                'airt/testing/activate_by_import.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../notebooks/API_Prediction.ipynb.

# %% auto 0
__all__ = ['Prediction']

# %% ../../notebooks/API_Prediction.ipynb 4
from typing import *

# %% ../../notebooks/API_Prediction.ipynb 5
import os
import requests

from pathlib import Path
import pandas as pd
from fastcore.foundation import patch
from tqdm import tqdm
import textwrap

from airt.logger import get_logger, set_level
from airt.helper import (
    get_data,
    post_data,
    delete_data,
    add_ready_column,
    generate_df,
    get_attributes_from_instances,
    add_example_to_docs,
)

from airt.components.client import Client
from airt.components.progress_status import ProgressStatus
from airt.constant import CLIENT_DB_USERNAME, CLIENT_DB_PASSWORD

# %% ../../notebooks/API_Prediction.ipynb 7
logger = get_logger(__name__)

# %% ../../notebooks/API_Prediction.ipynb 10
class Prediction(ProgressStatus):
    """A class to manage and download the predictions.

    The **Prediction** class is automatically instantiated by calling the `Model.predict` method of a `Model` instance.
    Currently, it is the only way to instantiate this class.

    At the moment, the prediction results can only be

    - downloaded to a local folder in parquet file format

    - pushed to Azure Blob Storage or an AWS S3 bucket

    - pushed to MySql or ClickHouse database

    We intend to support additional databases and storage mediums in future releases.
    """

    BASIC_PRED_COLS = ["uuid", "created", "total_steps", "completed_steps"]
    ALL_PRED_COLS = BASIC_PRED_COLS + [
        "model",
        "datasource",
        "region",
        "cloud_provider",
        "error",
    ]

    COLS_TO_RENAME = {
        "uuid": "prediction_uuid",
        "datasource": "datasource_uuid",
        "model": "model_uuid",
    }

    def __init__(
        self,
        uuid: str,
        datasource: Optional[str] = None,
        model: Optional[str] = None,
        created: Optional[str] = None,
        total_steps: Optional[int] = None,
        completed_steps: Optional[int] = None,
        region: Optional[str] = None,
        cloud_provider: Optional[str] = None,
        error: Optional[str] = None,
        disabled: Optional[bool] = None,
    ):
        """Constructs a new **Prediction** instance

        Warning:
            Do not construct this object directly by calling the constructor, instead please use
            `Model.predict` method of the Model instance.

        Args:
            uuid: Prediction uuid.
            datasource: DataSource uuid.
            model: Model uuid.
            created: Prediction creation date.
            total_steps: No of steps needed to complete the model prediction.
            completed_steps: No of steps completed so far in the model prediction.
            region: The region name of the cloud provider where the prediction is stored.
            cloud_provider: The name of the cloud storage provider where the prediction is stored.
            error: Contains the error message if running the predictions fails.
            disabled: A flag that indicates the prediction's status. If the prediction is deleted, then **False** will be set.
        """
        self.uuid = uuid
        self.datasource = datasource
        self.model = model
        self.created = created
        self.total_steps = total_steps
        self.completed_steps = completed_steps
        self.region = region
        self.cloud_provider = cloud_provider
        self.error = error
        self.disabled = disabled
        ProgressStatus.__init__(self, relative_url=f"/prediction/{self.uuid}")

    @staticmethod
    def _download_prediction_file_to_local(
        file_name: str, url: str, path: Union[str, Path]
    ) -> None:
        """Download the file to local directory.

        Args:
            file_name: Name of the file
            url: Url of the file
            path: Local directory path

        Raises:
            HTTPError: If the **url** is invalid or not reachable.
        """
        try:
            response = requests.get(url)
            response.raise_for_status()

        except requests.exceptions.HTTPError as e:
            raise requests.exceptions.HTTPError(e)

        else:
            with open(Path(path) / file_name, "wb") as f:
                f.write(response.content)

    @staticmethod
    def ls(
        offset: int = 0,
        limit: int = 100,
        disabled: bool = False,
        completed: bool = False,
    ) -> List["Prediction"]:
        """Return the list of Prediction instances available in the server.

        Args:
            offset: The number of predictions to offset at the beginning. If None, then the default value **0** will be used.
            limit: The maximum number of predictions to return from the server. If None,
                then the default value **100** will be used.
            disabled: If set to **True**, then only the deleted predictions will be returned. Else, the default value
                **False** will be used to return only the list of active predictions.
            completed: If set to **True**, then only the predictions that are successfully processed in server will be returned.
                Else, the default value **False** will be used to return all the predictions.

        Returns:
            A list of Prediction instances available in the server.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.
        """
        lists = Client._get_data(
            relative_url=f"/prediction/?disabled={disabled}&completed={completed}&offset={offset}&limit={limit}"
        )

        predx = [
            Prediction(
                uuid=pred["uuid"],
                model=pred["model"],
                datasource=pred["datasource"],
                created=pred["created"],
                total_steps=pred["total_steps"],
                completed_steps=pred["completed_steps"],
                region=pred["region"],
                cloud_provider=pred["cloud_provider"],
                error=pred["error"],
                disabled=pred["disabled"],
            )
            for pred in lists
        ]

        return predx

    @staticmethod
    def as_df(predx: List["Prediction"]) -> pd.DataFrame:
        """Return the details of prediction instances as a pandas dataframe.

        Args:
            predx: List of prediction instances.

        Returns:
            Details of all the prediction in a dataframe.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.
        """
        response = get_attributes_from_instances(predx, Prediction.BASIC_PRED_COLS)  # type: ignore

        df = generate_df(response, Prediction.BASIC_PRED_COLS)

        df = df.rename(columns=Prediction.COLS_TO_RENAME)

        return add_ready_column(df)

    def details(self) -> pd.DataFrame:
        raise NotImplementedError()

    def to_pandas(self) -> pd.DataFrame:
        raise NotImplementedError()

    def delete(self) -> pd.DataFrame:
        raise NotImplementedError()

    def to_s3(
        self,
        uri: str,
        access_key: Optional[str] = None,
        secret_key: Optional[str] = None,
    ) -> ProgressStatus:
        raise NotImplementedError()

    def to_azure_blob_storage(
        self,
        uri: str,
        credential: Optional[str] = None,
    ) -> ProgressStatus:
        raise NotImplementedError()

    def to_local(
        self,
        path: Union[str, Path],
        show_progress: Optional[bool] = True,
    ) -> None:
        raise NotImplementedError()

    def to_mysql(
        self,
        *,
        host: str,
        database: str,
        table: str,
        port: int = 3306,
        username: Optional[str] = None,
        password: Optional[str] = None,
    ) -> ProgressStatus:
        raise NotImplementedError()

    def to_clickhouse(
        self,
        *,
        host: str,
        database: str,
        table: str,
        port: int = 0,
        protocol: str,
        username: Optional[str] = None,
        password: Optional[str] = None,
    ) -> ProgressStatus:
        raise NotImplementedError()

# %% ../../notebooks/API_Prediction.ipynb 11
def _docstring_example():
    """
    Example:
        ```python
        # Importing necessary libraries
        import os
        import tempfile
        from datetime import timedelta

        from azure.identity import DefaultAzureCredential
        from azure.mgmt.storage import StorageManagementClient

        from  airt.client import Client, DataBlob, DataSource, Model, Prediction

        # Authenticate
        Client.get_token(username="{fill in username}", password="{fill in password}")

        # Create a datablob
        # In this example, the datablob will be stored in an AWS S3 bucket. The
        # access_key and the secret_key are set in the AWS_ACCESS_KEY_ID and
        # AWS_SECRET_ACCESS_KEY environment variables, and the region is set to
        # eu-west-3; feel free to change the cloud provider and the region to
        # suit your needs.
        db = DataBlob.from_s3(
            uri="{fill in uri}",
            cloud_provider="aws",
            region="eu-west-3"
        )

        # Display the status in a progress bar
        db.progress_bar()

        # Create a datasource
        ds = db.to_datasource(
            file_type="{fill in file_type}",
            index_column="{fill in index_column}",
            sort_by="{fill in sort_by}",
        )

        # Display the status in a progress bar
        ds.progress_bar()

        # Train a model to predicts which users will perform a purchase
        # event ("*purchase") three hours before they actually do it.
        model = ds.train(
            client_column="{fill in client_column}",
            target_column="{fill in target_column}",
            target="*purchase",
            predict_after=timedelta(hours=3)
        )

        # Display the status in a progress bar
        model.progress_bar()

        # Run predictions
        prediction = model.predict()
        prediction.progress_bar()

        # Print the details of the newly created prediction
        print(prediction.details())

        # Get the list of all prediction instances created by the currently logged-in user
        predx = Prediction.ls()
        print(predx)

        # Display the details of the prediction instances in a pandas dataframe
        df = Prediction.as_df(predx)
        print(df)

        # Display the prediction results in a pandas DataFrame
        print(prediction.to_pandas())

        # Push the prediction results to an AWS S3 bucket
        s3_status = prediction.to_s3(uri="{fill in s3_target_uri}")

        # Push the prediction results to an Azure Blob Storage
        os.environ["AZURE_SUBSCRIPTION_ID"] = "{fill in azure_subscription_id}"
        os.environ["AZURE_CLIENT_ID"] = "{fill in azure_client_id}"
        os.environ["AZURE_CLIENT_SECRET"] = "{fill in azure_client_secret}"
        os.environ["AZURE_TENANT_ID"]= "{fill in azure_tenant_id}"
        azure_group_name = "{fill in azure_group_name}"
        azure_storage_account_name = "{fill in azure_storage_account_name}"
        azure_storage_client = StorageManagementClient(
            DefaultAzureCredential(), os.environ["AZURE_SUBSCRIPTION_ID"]
        )
        azure_storage_keys = azure_storage_client.storage_accounts.list_keys(
            azure_group_name, azure_storage_account_name
        )
        azure_storage_keys = {v.key_name: v.value for v in azure_storage_keys.keys}
        azure_credential = azure_storage_keys['key1']

        azure_status = prediction.to_azure_blob_storage(
            uri="{fill in azure_target_uri}",
            credential=azure_credential
        )

        # Push the prediction results to a MySQL database
        mysql_status = prediction.to_mysql(
            username="{fill in mysql_db_username}",
            password="{fill in mysql_db_password}",
            host="{fill in mysql_host}",
            database="{fill in mysql_database}",
            table="{fill in mysql_table}",
        )

        # Push the prediction results to a ClickHouse database
        clickhouse_status = prediction.to_clickhouse(
            username="{fill in clickhouse_db_username}",
            password="{fill in clickhouse_db_password}",
            host="{fill in clickhouse_host}",
            database="{fill in clickhouse_database}",
            table="{fill in clickhouse_table}",
            protocol="native",
        )

        # Download the predictions to a local directory
        # In this example, the prediction results are downloaded
        # to a temporary directory
        with tempfile.TemporaryDirectory(prefix="predictions_results_") as d:
            prediction.to_local(path=d)
            # Check the downloaded prediction files
            downloaded_files = sorted(list(os.listdir(d)))
            print(downloaded_files)


        # Check the status
        s3_status.wait()
        azure_status.progress_bar()
        mysql_status.progress_bar()
        clickhouse_status.progress_bar()

        # Delete the prediction
        prediction.delete()
        ```
    """
    pass

# %% ../../notebooks/API_Prediction.ipynb 13
add_example_to_docs(Prediction, _docstring_example.__doc__)  # type: ignore
add_example_to_docs(Prediction.ls, _docstring_example.__doc__)  # type: ignore
add_example_to_docs(Prediction.as_df, _docstring_example.__doc__)  # type: ignore

# %% ../../notebooks/API_Prediction.ipynb 17
@patch
def details(self: Prediction) -> pd.DataFrame:
    """Return the details of a prediction.

    Returns:
        A pandas DataFrame encapsulating the details of the prediction.

    Raises:
        ConnectionError: If the server address is invalid or not reachable.
    """
    response = Client._get_data(relative_url=f"/prediction/{self.uuid}")

    df = pd.DataFrame(response, index=[0])[Prediction.ALL_PRED_COLS]

    df = df.rename(columns=Prediction.COLS_TO_RENAME)

    return add_ready_column(df)

# %% ../../notebooks/API_Prediction.ipynb 18
add_example_to_docs(Prediction.details, _docstring_example.__doc__)  # type: ignore

# %% ../../notebooks/API_Prediction.ipynb 21
@patch
def delete(self: Prediction) -> pd.DataFrame:
    """Delete a prediction from the server.

    Returns:
        A pandas DataFrame encapsulating the details of the deleted prediction.

    Raises:
        ConnectionError: If the server address is invalid or not reachable.
    """
    response = Client._delete_data(relative_url=f"/prediction/{self.uuid}")

    df = pd.DataFrame(response, index=[0])[Prediction.BASIC_PRED_COLS]

    df = df.rename(columns=Prediction.COLS_TO_RENAME)

    return add_ready_column(df)

# %% ../../notebooks/API_Prediction.ipynb 22
add_example_to_docs(Prediction.delete, _docstring_example.__doc__)  # type: ignore

# %% ../../notebooks/API_Prediction.ipynb 28
@patch
def to_pandas(self: Prediction) -> pd.DataFrame:
    """Return the prediction results as a pandas DataFrame

    Returns:
        A pandas DataFrame encapsulating the results of the prediction.

    Raises:
        ConnectionError: If the server address is invalid or not reachable.
    """
    response = Client._get_data(relative_url=f"/prediction/{self.uuid}/pandas")
    keys = list(response.keys())
    keys.remove("Score")
    index_name = keys[0]
    return (
        pd.DataFrame(response)
        .set_index(index_name)
        .sort_values("Score", ascending=False)
    )

# %% ../../notebooks/API_Prediction.ipynb 29
add_example_to_docs(Prediction.to_pandas, _docstring_example.__doc__)  # type: ignore

# %% ../../notebooks/API_Prediction.ipynb 31
@patch
def to_s3(
    self: Prediction,
    uri: str,
    access_key: Optional[str] = None,
    secret_key: Optional[str] = None,
) -> ProgressStatus:
    """Push the prediction results to the target AWS S3 bucket.

    Args:
        uri: Target S3 bucket uri.
        access_key: Access key for the target S3 bucket. If **None** (default value), then the value
            from **AWS_ACCESS_KEY_ID** environment variable is used.
        secret_key: Secret key for the target S3 bucket. If **None** (default value), then the value
            from **AWS_SECRET_ACCESS_KEY** environment variable is used.

    Returns:
        An instance of `ProgressStatus` class.

    Raises:
        ConnectionError: If the server address is invalid or not reachable.
    """
    access_key = (
        access_key if access_key is not None else os.environ["AWS_ACCESS_KEY_ID"]
    )
    secret_key = (
        secret_key if secret_key is not None else os.environ["AWS_SECRET_ACCESS_KEY"]
    )

    response = Client._post_data(
        relative_url=f"/prediction/{self.uuid}/to_s3",
        json=dict(uri=uri, access_key=access_key, secret_key=secret_key),
    )

    return ProgressStatus(relative_url=f"/prediction/push/{response['uuid']}")

# %% ../../notebooks/API_Prediction.ipynb 32
add_example_to_docs(Prediction.to_s3, _docstring_example.__doc__)  # type: ignore

# %% ../../notebooks/API_Prediction.ipynb 35
@patch
def to_azure_blob_storage(
    self: Prediction,
    uri: str,
    credential: str,
) -> ProgressStatus:
    """Push the prediction results to the target Azure Blob Storage.

    Args:
        uri: Target Azure Blob Storage uri.
        credential: Credential to access the Azure Blob Storage.

    Returns:
        An instance of `ProgressStatus` class.

    Raises:
        ConnectionError: If the server address is invalid or not reachable.
    """
    response = Client._post_data(
        relative_url=f"/prediction/{self.uuid}/to_azure_blob_storage",
        json=dict(uri=uri, credential=credential),
    )

    return ProgressStatus(relative_url=f"/prediction/push/{response['uuid']}")

# %% ../../notebooks/API_Prediction.ipynb 36
add_example_to_docs(Prediction.to_azure_blob_storage, _docstring_example.__doc__)  # type: ignore

# %% ../../notebooks/API_Prediction.ipynb 39
@patch
def to_local(
    self: Prediction,
    path: Union[str, Path],
    show_progress: Optional[bool] = True,
) -> None:
    """Download the prediction results to a local directory.

    Args:
        path: Local directory path.
        show_progress: Flag to set the progressbar visibility. If not passed, then the default value **True** will be used.

    Raises:
        FileNotFoundError: If the **path** is invalid.
        HTTPError: If the presigned AWS s3 uri to download the prediction results are invalid or not reachable.
    """
    response = Client._get_data(relative_url=f"/prediction/{self.uuid}/to_local")

    # Initiate progress bar
    t = tqdm(total=len(response), disable=not show_progress)

    for file_name, url in response.items():
        Prediction._download_prediction_file_to_local(file_name, url, Path(path))
        t.update()

    t.close()

# %% ../../notebooks/API_Prediction.ipynb 40
add_example_to_docs(Prediction.to_local, _docstring_example.__doc__)  # type: ignore

# %% ../../notebooks/API_Prediction.ipynb 43
@patch
def to_mysql(
    self: Prediction,
    *,
    host: str,
    database: str,
    table: str,
    port: int = 3306,
    username: Optional[str] = None,
    password: Optional[str] = None,
) -> ProgressStatus:
    """Push the prediction results to a mysql database.

    If the database requires authentication, pass the username/password as parameters or store it in
    the **AIRT_CLIENT_DB_USERNAME** and **AIRT_CLIENT_DB_PASSWORD** environment variables.

    Args:
        host: Database host name.
        database: Database name.
        table: Table name.
        port: Host port number. If not passed, then the default value **3306** will be used.
        username: Database username. If not passed, then the value set in the environment variable
            **AIRT_CLIENT_DB_USERNAME** will be used else the default value "root" will be used.
        password: Database password. If not passed, then the value set in the environment variable
            **AIRT_CLIENT_DB_PASSWORD** will be used else the default value "" will be used.

    Returns:
        An instance of `ProgressStatus` class.

    Raises:
        ConnectionError: If the server address is invalid or not reachable.
    """
    username = (
        username if username is not None else os.environ.get(CLIENT_DB_USERNAME, "root")
    )

    password = (
        password if password is not None else os.environ.get(CLIENT_DB_PASSWORD, "")
    )

    req_json = dict(
        host=host,
        port=port,
        username=username,
        password=password,
        database=database,
        table=table,
    )

    response = Client._post_data(
        relative_url=f"/prediction/{self.uuid}/to_mysql", json=req_json
    )

    return ProgressStatus(relative_url=f"/prediction/push/{response['uuid']}")

# %% ../../notebooks/API_Prediction.ipynb 44
add_example_to_docs(Prediction.to_mysql, _docstring_example.__doc__)  # type: ignore

# %% ../../notebooks/API_Prediction.ipynb 47
@patch
def to_clickhouse(
    self: Prediction,
    *,
    host: str,
    database: str,
    table: str,
    protocol: str,
    port: int = 0,
    username: Optional[str] = None,
    password: Optional[str] = None,
) -> ProgressStatus:
    """Push the prediction results to a clickhouse database.

    If the database requires authentication, pass the username/password as parameters or store it in
    the **CLICKHOUSE_USERNAME** and **CLICKHOUSE_PASSWORD** environment variables.

    Args:
        host: Remote database host name.
        database: Database name.
        table: Table name.
        protocol: Protocol to use (native/http).
        port: Host port number. If not passed, then the default value **0** will be used.
        username: Database username. If not passed, then the value set in the environment variable
            **CLICKHOUSE_USERNAME** will be used else the default value "root" will be used.
        password: Database password. If not passed, then the value set in the environment variable
            **CLICKHOUSE_PASSWORD** will be used else the default value "" will be used.

    Returns:
        An instance of `ProgressStatus` class.

    Raises:
        ConnectionError: If the server address is invalid or not reachable.
    """
    username = (
        username
        if username is not None
        else os.environ.get("CLICKHOUSE_USERNAME", "root")
    )

    password = (
        password if password is not None else os.environ.get("CLICKHOUSE_PASSWORD", "")
    )

    req_json = dict(
        host=host,
        database=database,
        table=table,
        protocol=protocol,
        port=port,
        username=username,
        password=password,
    )

    response = Client._post_data(
        relative_url=f"/prediction/{self.uuid}/to_clickhouse", json=req_json
    )

    return ProgressStatus(relative_url=f"/prediction/push/{response['uuid']}")

# %% ../../notebooks/API_Prediction.ipynb 48
add_example_to_docs(Prediction.to_clickhouse, _docstring_example.__doc__)  # type: ignore



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../notebooks/CLI_Model.ipynb.

# %% auto 0
__all__ = ['logger']

# %% ../../notebooks/CLI_Model.ipynb 3
from typing import *

# %% ../../notebooks/CLI_Model.ipynb 4
import os
import typer

from typer import echo
from tabulate import tabulate
import pandas as pd

from airt.cli import helper
from airt.logger import get_logger, set_level

# %% ../../notebooks/CLI_Model.ipynb 6
app = typer.Typer(
    help="A set of commands for querying the model training, evaluation, and prediction status."
)

# %% ../../notebooks/CLI_Model.ipynb 8
logger = get_logger(__name__)

# %% ../../notebooks/CLI_Model.ipynb 15
@app.command()
@helper.display_formated_table
@helper.requires_auth_token
def ls(
    offset: int = typer.Option(
        0,
        "--offset",
        "-o",
        help="The number of models to offset at the beginning. If None, then the default value **0** will be used.",
    ),
    limit: int = typer.Option(
        100,
        "--limit",
        "-l",
        help="The maximum number of models to return from the server. If None, "
        "then the default value **100** will be used.",
    ),
    disabled: bool = typer.Option(
        False,
        "--disabled",
        help="If set to **True**, then only the deleted models will be returned. Else, the default value "
        "**False** will be used to return only the list of active models.",
    ),
    completed: bool = typer.Option(
        False,
        "--completed",
        help="If set to **True**, then only the models that are successfully downloaded "
        "to the server will be returned. Else, the default value **False** will be used to "
        "return all the models.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output only uuids of model separated by space",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Return the list of models."""

    from airt.client import Model

    mx = Model.ls(offset=offset, limit=limit, disabled=disabled, completed=completed)

    df = Model.as_df(mx)

    df["created"] = helper.humanize_date(df["created"])

    return {"df": df, "quite_column_name": "model_uuid"}

# %% ../../notebooks/CLI_Model.ipynb 21
@app.command()
@helper.display_formated_table
@helper.requires_auth_token
def details(
    uuid: str = typer.Argument(
        ...,
        help="Model uuid",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Return the details of a model."""

    from airt.client import Model

    model = Model(uuid=uuid)
    df = model.details()

    df["created"] = helper.humanize_date(df["created"])

    return {"df": df, "quite_column_name": "model_uuid"}

# %% ../../notebooks/CLI_Model.ipynb 25
@app.command()
@helper.display_formated_table
@helper.requires_auth_token
def rm(
    uuid: str = typer.Argument(
        ...,
        help="Model uuid",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the deleted Model uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Delete a model from the server."""

    from airt.client import Model

    model = Model(uuid=uuid)
    df = model.delete()

    df["created"] = helper.humanize_date(df["created"])

    return {"df": df, "quite_column_name": "model_uuid"}

# %% ../../notebooks/CLI_Model.ipynb 29
@app.command()
@helper.requires_auth_token
def predict(
    data_uuid: str = typer.Option(
        ...,
        "--data_uuid",
        help="DataSource uuid.",
    ),
    uuid: str = typer.Option(
        ...,
        "--uuid",
        help="Model uuid.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the prediction id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Run predictions against the trained model."""

    from airt.client import Model

    model = Model(uuid=uuid)

    prediction = model.predict(data_uuid=data_uuid)

    if quiet:
        prediction.wait()

        typer.echo(f"{prediction.uuid}")
    else:
        typer.echo(f"Running predictions for prediction uuid: {prediction.uuid}")
        prediction.progress_bar()

# %% ../../notebooks/CLI_Model.ipynb 33
@app.command()
@helper.requires_auth_token
def evaluate(
    uuid: str = typer.Argument(
        ...,
        help="Model uuid.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Return the evaluation metrics of the trained model.

    Currently, this command returns the model's accuracy, precision, and recall. In the future, more performance metrics will be added.
    """

    from airt.client import Model

    model = Model(uuid=uuid)

    df = model.evaluate()

    typer.echo(tabulate(df, headers="keys", tablefmt="plain"))  # type: ignore



# AUTOGENERATED! DO NOT EDIT! File to edit: ../notebooks/Client.ipynb.

# %% auto 0
__all__ = ['Client', 'DataSource', 'DataBlob', 'ProgressStatus', 'Model', 'Prediction', 'User', 'APIKey']

# %% ../notebooks/Client.ipynb 2
from .components.client import Client as _Client
from .components.datasource import DataSource as _DataSource
from .components.datablob import DataBlob as _DataBlob
from .components.progress_status import ProgressStatus as _ProgressStatus
from .components.model import Model as _Model
from .components.prediction import Prediction as _Prediction
from .components.user import User as _User
from .components.api_key import APIKey as _APIKey


Client = _Client
DataSource = _DataSource
DataBlob = _DataBlob
ProgressStatus = _ProgressStatus
Model = _Model
Prediction = _Prediction
User = _User
APIKey = _APIKey

for cls in [
    Client,
    DataSource,
    DataBlob,
    ProgressStatus,
    Model,
    Prediction,
    User,
    APIKey,
]:
    cls.__module__ = "airt.client"



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../notebooks/CLI_Version.ipynb.

# %% auto 0
__all__ = ['logger']

# %% ../../notebooks/CLI_Version.ipynb 3
from typing import *

# %% ../../notebooks/CLI_Version.ipynb 4
import typer
import logging
from tabulate import tabulate
import pandas as pd

from airt.client import Client
from airt.logger import get_logger, set_level

# %% ../../notebooks/CLI_Version.ipynb 6
app = typer.Typer()

# %% ../../notebooks/CLI_Version.ipynb 8
logger = get_logger(__name__)

# %% ../../notebooks/CLI_Version.ipynb 11
def version() -> None:
    """Return the server and client versions."""
    try:
        df = pd.Series(Client.version(), name="Version").to_frame()

        typer.echo(tabulate(df, headers="keys", tablefmt="plain"))  # type: ignore

    except Exception as e:
        typer.echo(message=f"Error: {e}", err=True)
        raise typer.Exit(code=1)

# %% ../../notebooks/CLI_Version.ipynb 12
app.command()(version)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../notebooks/CLI_User.ipynb.

# %% auto 0
__all__ = ['logger']

# %% ../../notebooks/CLI_User.ipynb 3
from typing import *

# %% ../../notebooks/CLI_User.ipynb 4
import os

import typer
from typer import echo
from tabulate import tabulate
import datetime as dt
import pandas as pd
import qrcode

from airt.client import Client
from airt.cli import helper
from airt.logger import get_logger, set_level
from airt.constant import SERVICE_PASSWORD

# %% ../../notebooks/CLI_User.ipynb 6
app = typer.Typer(
    help="A set of commands for managing users and their authentication in the server."
)

# %% ../../notebooks/CLI_User.ipynb 8
logger = get_logger(__name__)

# %% ../../notebooks/CLI_User.ipynb 16
@app.command()
@helper.display_formated_table
@helper.requires_auth_token
def details(
    user: Optional[str] = typer.Option(
        None,
        "--user",
        "-u",
        help="Account user_uuid/username to get details. If not passed, then the currently logged-in details will be returned.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output user uuid only.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Get user details

    Please do not pass the optional 'user' option unless you are a super user. Only a super user can view details for other users.
    """

    from airt.client import User

    df = pd.DataFrame(User.details(user=user), index=[0])[User.USER_COLS]

    return {"df": df}

# %% ../../notebooks/CLI_User.ipynb 22
@app.command()
@helper.display_formated_table
@helper.requires_totp()
@helper.requires_auth_token
def create(
    username: str = typer.Option(
        ...,
        "--username",
        "-un",
        help="The new user's username. The username must be unique or an exception will be thrown.",
    ),
    first_name: str = typer.Option(
        ...,
        "--first_name",
        "-fn",
        help="The new user's first name.",
    ),
    last_name: str = typer.Option(
        ...,
        "--last_name",
        "-ln",
        help="The new user's last name.",
    ),
    email: str = typer.Option(
        ...,
        "--email",
        "-e",
        help="The new user's email. The email must be unique or an exception will be thrown.",
    ),
    password: str = typer.Option(
        ...,
        "--password",
        "-p",
        help="The new user's password.",
    ),
    subscription_type: str = typer.Option(
        ...,
        "--subscription_type",
        "-st",
        help="User subscription type. Currently, the API supports only the following subscription types **small**, **medium** and **large**.",
    ),
    super_user: bool = typer.Option(
        False,
        "--super_user",
        "-su",
        help="If set to **True**, then the new user will have super user privilages. If **None**, then the default value "
        "**False** will be used to create a non-super user.",
    ),
    phone_number: Optional[str] = typer.Option(
        None,
        "--phone_number",
        "-ph",
        help="Phone number to be added to the user account. The phone number should follow the pattern of the country "
        "code followed by your phone number. For example, 440123456789, +440123456789, 00440123456789, +44 0123456789,"
        "and (+44) 012 345 6789 are all valid formats for registering a UK phone number.",
    ),
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app. Please pass this optional argument only if you have activated the MFA for your account.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output user uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Create a new user in the server."""

    from airt.client import User

    df = User.create(
        username=username,
        first_name=first_name,
        last_name=last_name,
        email=email,
        subscription_type=subscription_type,
        super_user=super_user,
        password=password,
        phone_number=phone_number,
        otp=otp,
    )

    df["created"] = helper.humanize_date(df["created"])

    return {"df": df}

# %% ../../notebooks/CLI_User.ipynb 25
@app.command()
@helper.display_formated_table
@helper.requires_auth_token
def ls(
    offset: int = typer.Option(
        0,
        "--offset",
        "-o",
        help="The number of users to offset at the beginning. If **None**, then the default value **0** will be used.",
    ),
    limit: int = typer.Option(
        100,
        "--limit",
        "-l",
        help="The maximum number of users to return from the server. If None, then the default value 100 will be used.",
    ),
    disabled: bool = typer.Option(
        False,
        "--disabled",
        help="If set to **True**, then only the deleted users will be returned. Else, the default value **False** will "
        "be used to return only the list of active users.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output only user uuids separated by space",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Return the list of users available in the server."""

    from airt.client import User

    ux = User.ls(offset=offset, limit=limit, disabled=disabled)

    df = User.as_df(ux)

    df["created"] = helper.humanize_date(df["created"])

    return {"df": df}

# %% ../../notebooks/CLI_User.ipynb 29
@app.command()
@helper.display_formated_table
@helper.requires_totp()
@helper.requires_auth_token
def disable(
    users: List[str] = typer.Argument(
        ...,
        help="user_uuid/username to disabled.  To disable multiple users, please pass the uuids/names separated by space.",
    ),
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app. Please pass this optional argument only if you have activated the MFA for your account.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output user uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Disable a user in the server."""

    from airt.client import User

    users = [user for user in users]
    #     formated_users = helper.separate_integers_and_strings(users)

    df = User.disable(user=users, otp=otp)  # type: ignore
    df["created"] = helper.humanize_date(df["created"])

    return {"df": df}

# %% ../../notebooks/CLI_User.ipynb 33
@app.command()
@helper.display_formated_table
@helper.requires_totp()
@helper.requires_auth_token
def enable(
    users: List[str] = typer.Argument(
        ...,
        help="user_uuid/username to enable. To enable multiple users, please pass the uuids/names separated by space.",
    ),
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app. Please pass this optional argument only if you have activated the MFA for your account.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output user uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Enable a disabled user in the server."""

    from airt.client import User

    users = [user for user in users]
    #     formated_users = helper.separate_integers_and_strings(users)

    df = User.enable(user=users, otp=otp)  # type: ignore
    df["created"] = helper.humanize_date(df["created"])

    return {"df": df}

# %% ../../notebooks/CLI_User.ipynb 39
@app.command()
@helper.display_formated_table
@helper.requires_totp()
@helper.requires_auth_token
def update(
    user: Optional[str] = typer.Option(
        None,
        "--user",
        help="Account user_uuid/username to update. If not passed, then the default value None will be used to update the currently logged-in user details.",
    ),
    username: Optional[str] = typer.Option(
        None,
        "--username",
        "-un",
        help="New username for the user.",
    ),
    first_name: Optional[str] = typer.Option(
        None,
        "--first_name",
        "-fn",
        help="New first name for the user.",
    ),
    last_name: Optional[str] = typer.Option(
        None,
        "--last_name",
        "-ln",
        help="New last name for the user.",
    ),
    email: Optional[str] = typer.Option(
        None,
        "--email",
        "-e",
        help="New email for the user.",
    ),
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app. Please pass this optional argument only if you have activated the MFA for your account.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output user uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Update existing user details in the server.

    Please do not pass the optional user option unless you are a super user. Only a
    super user can update details for other users.

    """

    from airt.client import User

    df = User.update(
        user=user,
        username=username,
        first_name=first_name,
        last_name=last_name,
        email=email,
        otp=otp,
    )

    df["created"] = helper.humanize_date(df["created"])

    return {"df": df}

# %% ../../notebooks/CLI_User.ipynb 43
@app.command("register-phone-number")
@helper.requires_totp()
@helper.requires_auth_token
def register_phone_number(
    phone_number: Optional[str] = typer.Option(
        None,
        "--phone-number",
        "-p",
        help="""Phone number to register. The phone number should follow the pattern of the 
            country code followed by your phone number. For example, **440123456789, +440123456789,
            00440123456789, +44 0123456789, and (+44) 012 345 6789** are all valid formats for registering a UK phone number.
            If the phone number is not passed in the arguments, then the OTP will be sent to the phone 
            number that was already registered to the user's account.""",
    ),
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app. Please pass this optional argument only if you have activated the MFA for your account.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Register and validate a phone number

    This is an interactive command, one called it will send an OTP via SMS to the phone number. Please enter the OTP you have received
    in the interactive prompt to complete the phone number registration process.

    After ten invalid OTP attempts, you have to call this command again to register the phone number.
    """

    from airt.client import User

    user = User.register_phone_number(phone_number=phone_number, otp=otp)

    while True:
        try:
            sms_otp = typer.prompt(
                f"We have sent a One-Time Password (OTP) to the phone number {user['phone_number']}. Please enter here"
            )
            typer.echo("\n")
            response = User.validate_phone_number(otp=sms_otp)
            typer.echo(
                f"The phone number {user['phone_number']} is successfully registered. We will send the OTP via SMS to this registered phone number when requested."
            )
            break
        except ValueError as e:
            typer.echo(e)
            if ("Too many failed attempts" in str(e)) or (
                "OTP entered is expired" in str(e)
            ):
                break

# %% ../../notebooks/CLI_User.ipynb 45
@app.command("reset-password")
def reset_password(
    username: Optional[str] = typer.Option(
        None, "--username", "-u", help="Account username to reset the password"
    ),
    new_password: Optional[str] = typer.Option(
        None, "--new-password", "-np", help="New password to set for the account"
    ),
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Reset the account password

    We currently support two types of OTPs to reset the password for your account and you don't have to be logged in to call this command

    \n\nThe command switches to interactive mode unless all arguments are passed. The interactive mode will prompt you for the missing details and ask you to choose a recovery option to reset your password. Currently, we only support resetting the password either using a TOTP or SMS OTP.
    \n\nIf you have already activated the MFA for your account, then you can either enter the dynamically generated six-digit verification code from the authenticator app (TOTP) or request an OTP via SMS to your registered phone number.
    \n\nIf the MFA is not activated already, then you can only request the OTP via SMS to your registered phone number.
    \n\nAfter selecting an option, please follow the on-screen instructions to reset your password. In case, you don't have MFA enabled or don't have access to your registered phone number, please contact your administrator.
    """

    from airt.client import User

    if username is None:
        username = typer.prompt("Please enter your username")

    if new_password is None:
        new_password = typer.prompt(
            "Please enter your new password", hide_input=True, confirmation_prompt=True
        )

    if otp is not None:
        try:
            status = User.reset_password(
                username=username, new_password=new_password, otp=otp
            )
            typer.echo(f"\n{status}")
            typer.echo(
                f"\nPlease don't forget to set the updated password in the `{SERVICE_PASSWORD}` environment variable"
            )
        except ValueError as e:
            typer.echo(e)
            raise typer.Exit(code=1)
    else:
        typer.echo("\nPlease choose an option to reset your password\n\n")

        while True:
            typer.echo(
                "[1] Reset password using the dynamically generated six-digit verification code from the authenticator application\n"
            )
            typer.echo(
                "[2] Reset password by requesting the OTP via SMS to the registered phone number\n"
            )
            typer.echo(
                "If you cannot access the authenticator application or your registered phone number, please contact your administrator.\n"
            )

            recovery_option = typer.prompt("Password reset option")

            if recovery_option in ["1", "2"]:
                break
            typer.echo("Please enter a valid password reset option")

        if recovery_option == "1":
            try:
                totp = typer.prompt(
                    "Please enter the OTP displayed in the authenticator application"
                )
                status = User.reset_password(
                    username=username, new_password=new_password, otp=totp
                )
            except ValueError as e:
                typer.echo(e)

        elif recovery_option == "2":
            try:
                sms_status = User.send_sms_otp(
                    username=username, message_template_name="reset_password"
                )
                typer.echo(f"\n{sms_status}\n")
                sms_otp = typer.prompt(
                    f"Please enter the One-Time Password (OTP) you received on your registered phone number"
                )
                status = User.reset_password(
                    username=username, new_password=new_password, otp=sms_otp
                )
            except ValueError as e:
                typer.echo(e)

        else:
            raise typer.Exit(code=1)

        typer.echo(f"\n{status}")

        if os.environ.get(SERVICE_PASSWORD) is not None:

            typer.echo(
                f"\nPlease don't forget to set the updated password in the `{SERVICE_PASSWORD}` environment variable"
            )

# %% ../../notebooks/CLI_User.ipynb 48
mfa_app = typer.Typer(
    help="Commands for enabling and disabling Multi-Factor Authentication (MFA)."
)

# %% ../../notebooks/CLI_User.ipynb 49
@mfa_app.command()  # type: ignore
@helper.requires_totp()
@helper.requires_auth_token
def enable(
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app. Please pass this optional argument only if you have activated the MFA for your account.",
    ),
) -> None:
    """Enable Multi-Factor Authentication (MFA) for the user.

    This is an interactive command and will generate a QR code. You can use an authenticator app, such as Google Authenticator
    to scan the code and enter the valid six-digit verification code from the authenticator app in the interactive prompt to
    enable and activate MFA for your account.

    After three invalid attempts, you have to call this command again to generate a new QR code.
    """

    from airt.client import User

    qr = qrcode.QRCode()
    qr.add_data(User._get_mfa_provision_url(otp=otp))

    typer.echo("Please open an authenticator app and scan the QR code below:")
    #     typer.echo(qr.print_ascii(invert=True))
    typer.echo(qr.print_ascii())

    for i in range(3):

        try:
            activation_otp = typer.prompt(
                "Please enter the OTP displayed in the authenticator app"
            )
            response = User.activate_mfa(otp=activation_otp)
            typer.echo("Multi-Factor Authentication (MFA) successfully activated.")

            details = User.details()
            status = helper.get_phone_registration_status(details)
            if status is not None:
                typer.echo(status)

            break

        except ValueError as e:
            typer.echo(e)

# %% ../../notebooks/CLI_User.ipynb 51
@mfa_app.command()  # type: ignore
@helper.requires_totp_or_otp(message_template_name="disable_mfa")
@helper.requires_auth_token
def disable(
    user: Optional[str] = typer.Option(
        None,
        "--user",
        "-u",
        help="Account user_uuid/username to disable MFA. If not passed, then the default value None will be used to disable MFA for the currently logged-in user.",
    ),
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app or the OTP you have received via SMS.",
    ),
) -> None:
    """Disable Multi-Factor Authentication (MFA) for the user.

    The command switches to interactive mode unless the OTP argument is passed. The interactive mode will prompt you to
    choose an OTP option you want to use. Currently, we only support disabling MFA either using a TOTP or SMS OTP.

    If you have access to the authenticator application, then you can either enter the dynamically generated six-digit
    verification code from the authenticator app (TOTP) or request an OTP via SMS to your registered phone number.

    After selecting an option, please follow the on-screen instructions to disable MFA for your account. In case,
    you don't have access to the authenticator app and your registered phone number, please contact your administrator.

    Note: Please do not pass the user argument unless you are a super user. Only
    a super user can disable MFA for other users.
    """

    from airt.client import User

    User.disable_mfa(user=user, otp=otp)

    typer.echo(
        "Multi-Factor Authentication (MFA) is successfully deactivated for the user."
    )

# %% ../../notebooks/CLI_User.ipynb 55
# Adding mfa as a subcommand for user command
app.add_typer(mfa_app, name="mfa")

# %% ../../notebooks/CLI_User.ipynb 57
sso_app = typer.Typer(help="Commands for enabling and disabling Single sign-on (SSO).")

# %% ../../notebooks/CLI_User.ipynb 58
@sso_app.command()  # type: ignore
@helper.requires_totp()
@helper.requires_auth_token
def disable(
    sso_provider: str = typer.Argument(
        ...,
        help="Name of the Single sign-on (SSO) identity provider. At present, the API only supports Google and Github as valid SSO identity providers.",
    ),
    user: Optional[str] = typer.Option(
        None,
        "--user",
        "-u",
        help="Account user_uuid/username to disable MFA. If not passed, then the default value None will be used to disable SSO for the currently logged-in user.",
    ),
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app. Please pass this optional argument only if you have activated the MFA for your account.",
    ),
) -> None:
    """Disable Single sign-on (SSO) for the user.

    Please do not pass the user argument unless you are a super user. Only
    a super user can disable SSO for other users.
    """

    from airt.client import User

    success_msg = User.disable_sso(sso_provider=sso_provider, user=user, otp=otp)
    typer.echo(success_msg)

# %% ../../notebooks/CLI_User.ipynb 60
@sso_app.command()  # type: ignore
@helper.requires_totp()
@helper.requires_auth_token
def enable(
    sso_provider: str = typer.Argument(
        ...,
        help="Name of the Single sign-on (SSO) identity provider. At present, the API only supports **Google** and **Github** as valid SSO identity providers.",
    ),
    sso_email: str = typer.Option(
        ...,
        "--email",
        "-e",
        help="Email id going to be used for SSO authentication.",
    ),
    otp: Optional[str] = typer.Option(
        None,
        "--otp",
        help="Dynamically generated six-digit verification code from the authenticator app. Please pass this optional argument only if you have activated the MFA for your account.",
    ),
) -> None:
    """Enable Single sign-on (SSO) for the user"""

    from airt.client import User

    success_msg = User.enable_sso(
        sso_provider=sso_provider, sso_email=sso_email, otp=otp
    )

    typer.echo(success_msg)

# %% ../../notebooks/CLI_User.ipynb 63
# Adding sso as a subcommand for user command
app.add_typer(sso_app, name="sso")



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../notebooks/Docstring_helpers.ipynb.

# %% auto 0
__all__ = ['run_examples_from_docstring']

# %% ../../notebooks/Docstring_helpers.ipynb 3
from typing import *
import sys
import os

import re

import textwrap
from subprocess import run, CalledProcessError  # nosec: B404
from tempfile import TemporaryDirectory
from pathlib import Path

from rich import print
from rich.console import Group
from rich.panel import Panel

# %% ../../notebooks/Docstring_helpers.ipynb 7
def _extract_examples_from_docstring(o: Any) -> List[str]:
    try:
        import griffe
    except:
        raise Exception(
            "This function should only be used for testing where griffe package is installed."
        )

    if o.__doc__ is None:
        raise ValueError(f"{o.__name__}.__doc__ = {o.__doc__}")
    sections = griffe.docstrings.parse(
        griffe.dataclasses.Docstring(o.__doc__), griffe.docstrings.Parser.google
    )

    def find_python_code(s: str) -> str:
        code = [x[0] for x in re.findall("```python((\n|.)+?)```", s)]
        #         print(f"{code=}")
        if len(code) == 0:
            raise ValueError(f"No python code found in {s}")
        return "\n\n".join(code)

    examples = [
        find_python_code(section.value.description)  # type: ignore
        for section in sections
        if section.kind.value == "admonition" and section.value.annotation == "example"  # type: ignore
    ]

    return examples

# %% ../../notebooks/Docstring_helpers.ipynb 11
def _get_keywords(examples: List[str]) -> List[str]:
    keywords: List[str] = sum(
        [
            [x[9:-1] for x in re.findall("{fill in \w+}", example)]
            for example in examples
        ],
        start=[],
    )

    return keywords

# %% ../../notebooks/Docstring_helpers.ipynb 13
def _replace_keywords(examples: List[str], **kwargs) -> List[str]:
    keywords = _get_keywords(examples)

    if set(keywords) > set(kwargs.keys()):
        raise ValueError(f"{set(keywords)} > {set(kwargs.keys())}")

    for keyword in keywords:
        examples = [
            example.replace("{fill in " + keyword + "}", kwargs[keyword])
            for example in examples
        ]

    return examples

# %% ../../notebooks/Docstring_helpers.ipynb 15
def _format_output(
    s: str,
    *,
    title: str,
    supress: bool = False,
    sub_dict: Optional[Dict[str, str]] = None,
):
    if sub_dict:
        for pattern, replacement in sub_dict.items():
            s = re.sub(pattern, replacement, s)
    if supress:
        return Panel("", title=f"{title} supressed")
    else:
        return Panel(s, title=title)

# %% ../../notebooks/Docstring_helpers.ipynb 17
def run_examples_from_docstring(
    o: Any,
    *,
    supress_stdout: bool = False,
    supress_stderr: bool = False,
    sub_dict: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Runs example from a docstring

    Parses docstring of an objects looking for examples. The examples are then saved into files and executed
    in a separate process.

    Note:
        Execution context is not the same as the one in the notebook because we want examples to work from
        user code. Make sure you compiled the library prior to executing the examples, otherwise you might
        be running them agains an old version of the library.

    Args:
        o: an object, typically a function or a class, for which docstring is being parsed for examples
        supress_stdout: omit stdout from output, typically due to security considerations
        supress_stderr: omit stderr from output, typically due to security considerations
        sub_dict: a dictionary mapping regexp patterns into replacement strings used to mask stdout and
            stderr, typically used to mask sensitive information such as passwords

        **kwargs: arguments use to replace "{fill in **param**}" in docstring with the actual values when running examples

    Raises:
        ValueError: if some params are missing from the **kwargs**
        RuntimeException: if example fails

    Example:
        ```python

        def f():
            ```python
            Example:
                print("Hello {fill in name}!")
                print("Goodbye {fill in other_name}!")
            ```
            pass


        run_examples_from_docstring(f, name="John", other_name="Jane")
        ```
    """
    examples = _extract_examples_from_docstring(o)
    executable_examples = _replace_keywords(examples, **kwargs)
    for example, executable_example in zip(examples, executable_examples):
        with TemporaryDirectory() as d:
            cmd_path = (Path(d) / "example.py").absolute()
            with open(cmd_path, "w") as f:
                f.write(executable_example)
            process = run(  # nosec: B603
                [sys.executable, str(cmd_path)], capture_output=True, text=True
            )
            panel_group = Group(
                "Example:\n" + textwrap.indent(example, " " * 4),
                _format_output(
                    process.stdout,
                    title="stdout",
                    supress=supress_stdout,
                    sub_dict=sub_dict,
                ),
                _format_output(
                    process.stderr,
                    title="stderr",
                    supress=supress_stderr,
                    sub_dict=sub_dict,
                ),
            )
            print(Panel(panel_group))
            if process.returncode != 0:
                raise RuntimeError(process.stderr)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../notebooks/Sanitize_Secrets.ipynb.

# %% auto 0
__all__ = ['old_log', 'old_publish_display_data', 'sanitize_secrets', 'new_publish_display_data', 'sanitized_print']

# %% ../notebooks/Sanitize_Secrets.ipynb 2
import builtins
import logging
import re
import sys
from logging import Logger
from typing import *

import IPython

from .logger import get_logger

from fastcore.utils import patch

# %% ../notebooks/Sanitize_Secrets.ipynb 9
def sanitize_secrets(s: str) -> str:
    d = {
        "(A3T[A-Z0-9]|AKIA|AGPA|AIDA|AROA|AIPA|ANPA|ANVA|ASIA)\w+": "*" * 20,
        "AWS_ACCESS_KEY_ID':\s*'.*?'": "AWS_ACCESS_KEY_ID': " + f"'{'*' * 20}'",
        "AWSAccessKeyId':\s*'.*?'": "AWSAccessKeyId': " + f"'{'*' * 20}'",
        "KEY':\s*'.*?'": "KEY': " + f"'{'*' * 40}'",
        "KEY\s*=\s*'.*?'": "KEY = " + f"'{'*' * 40}'",
        "AZURE_SUBSCRIPTION_ID':\s*'.*?'": "AZURE_SUBSCRIPTION_ID': " + f"'{'*' * 36}'",
        "AZURE_TENANT_ID':\s*'.*?'": "AZURE_TENANT_ID': " + f"'{'*' * 36}'",
        "AZURE_CLIENT_ID':\s*'.*?'": "AZURE_CLIENT_ID': " + f"'{'*' * 36}'",
        "SECRET':\s*'.*?'": "SECRET': " + f"'{'*' * 40}'",
        "PASSWORD':\s*'.*?'": "PASSWORD': " + f"'{'*' * 40}'",
        "POLICY':\s*'.*?'": "POLICY': " + f"'{'*' * 252}'",
        "SIGNATURE':\s*'.*?'": "SIGNATURE': " + f"'{'*' * 28}'",
        "://.*@": "://" + "*" * 40 + "@",
        "value':\s*'[a-zA-Z0-9]{8}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{4}-[a-zA-Z0-9]{12}'": f"value': '{'*'*8}-{'*'*4}-{'*'*4}-{'*'*4}-{'*'*12}'",
        "[a-zA-Z0-9]{5}~[a-zA-Z0-9-]{34}": f"{'*'*40}",
    }

    d2 = {k.replace("'", '"'): v.replace("'", '"') for k, v in d.items() if "'" in k}
    for k, v in d2.items():
        d[k] = v

    d3 = {k.lower(): v.lower() for k, v in d.items()}
    for k, v in d3.items():
        d[k] = v

    d4 = {
        k.replace("':", "="): v.replace("':", " =") for k, v in d.items() if "':" in k
    }
    for k, v in d4.items():
        d[k] = v

    d = {k.replace("'", "\\\\?'"): v for k, v in d.items()}

    if not isinstance(s, str):
        s = s.__repr__()

    for pattern, replacement in d.items():
        s = re.sub(pattern, replacement, s)

    return s

# %% ../notebooks/Sanitize_Secrets.ipynb 17
old_log = Logger._log

# %% ../notebooks/Sanitize_Secrets.ipynb 18
@patch
def _log(self: Logger, level, msg, *args, **kwargs):
    return old_log(self, level, sanitize_secrets(msg), *args, **kwargs)

# %% ../notebooks/Sanitize_Secrets.ipynb 23
old_publish_display_data = IPython.core.display_functions.publish_display_data

# %% ../notebooks/Sanitize_Secrets.ipynb 24
def new_publish_display_data(
    data,
    metadata=None,
    source=IPython.core.display_functions._sentinel,
    *,
    transient=None,
    **kwargs
):
    sanitized_data = {
        k: sanitize_secrets(v.__repr__() if not isinstance(v, str) else v)
        for k, v in data.items()
    }
    return old_publish_display_data(
        sanitized_data, metadata=metadata, source=source, transient=transient, **kwargs
    )


IPython.core.display_functions.publish_display_data = new_publish_display_data

# %% ../notebooks/Sanitize_Secrets.ipynb 27
old_print = builtins.print

# %% ../notebooks/Sanitize_Secrets.ipynb 28
def sanitized_print(*objects, sep=" ", end="\n", file=sys.stdout, flush=False):
    new_objs = [
        sanitize_secrets(obj.__repr__() if not isinstance(obj, str) else obj)
        for obj in objects
    ]
    return old_print(*new_objs, sep=sep, end=end, file=file, flush=flush)


# builtins.print = new_print



# AUTOGENERATED! DO NOT EDIT! File to edit: ../notebooks/Constants.ipynb.

# %% auto 0
__all__ = ['CLIENT_DB_USERNAME', 'CLIENT_DB_PASSWORD', 'SERVICE_USERNAME', 'SERVICE_PASSWORD', 'SERVER_URL', 'SERVICE_TOKEN',
           'SERVICE_SUPER_USER', 'PROD_URL', 'CLIENT_NAME']

# %% ../notebooks/Constants.ipynb 2
# The below are the names of the envirnment variables and not actual values
CLIENT_DB_USERNAME = "AIRT_CLIENT_DB_USERNAME"
CLIENT_DB_PASSWORD = "AIRT_CLIENT_DB_PASSWORD"  # nosec B105
SERVICE_USERNAME = "AIRT_SERVICE_USERNAME"
SERVICE_PASSWORD = "AIRT_SERVICE_PASSWORD"  # nosec B105
SERVER_URL = "AIRT_SERVER_URL"
SERVICE_TOKEN = "AIRT_SERVICE_TOKEN"  # nosec B105
SERVICE_SUPER_USER = "AIRT_SERVICE_SUPER_USER"
PROD_URL = "https://api.airt.ai"
CLIENT_NAME = "airt"



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../notebooks/API_Client.ipynb.

# %% auto 0
__all__ = ['Client']

# %% ../../notebooks/API_Client.ipynb 4
from typing import *

# %% ../../notebooks/API_Client.ipynb 5
import os
import importlib
import urllib.parse
import json
import secrets

from fastcore.foundation import patch
import pandas as pd

import airt
from airt.logger import get_logger, set_level
from airt.helper import get_data, post_data, delete_data, get_base_url
from airt.constant import (
    SERVICE_USERNAME,
    SERVICE_PASSWORD,
    SERVER_URL,
    CLIENT_NAME,
    SERVICE_TOKEN,
)

# %% ../../notebooks/API_Client.ipynb 7
logger = get_logger(__name__)

# %% ../../notebooks/API_Client.ipynb 10
def _get_credentials(
    username: Optional[str] = None, password: Optional[str] = None
) -> Tuple[(str, str)]:
    """Returns the value for username and password.

    If username is **None**, retrive the value from AIRT_SERVICE_USERNAME environment variable.
    If password is **None**, retrive the value from AIRT_SERVICE_PASSWORD environment variable.

    Args:
        username: Username for your developer account.
        password: Password for your developer account.

    Returns:
        The values for username and password as a tuple.

    Raises:
        Key Error, if the environment variables are not set.
    """

    username = username if username is not None else os.environ.get(SERVICE_USERNAME)

    password = password if password is not None else os.environ.get(SERVICE_PASSWORD)

    if not username and not password:
        raise KeyError(
            f"The username and password are neither passed as parameters nor set in the environment variables "
            f"`{SERVICE_USERNAME}` and `{SERVICE_PASSWORD}`."
        )

    elif not username:
        raise KeyError(
            f"The username is neither passed as parameter nor set in the environment variable {SERVICE_USERNAME}."
        )

    elif not password:
        raise KeyError(
            f"The password is neither passed as parameter nor set in the environment variable {SERVICE_PASSWORD}."
        )

    return (username, password)

# %% ../../notebooks/API_Client.ipynb 15
class Client:
    """A class for authenticating and accessing the airt service.

    To access the airt service, you must first create a developer account. To obtain one, please contact us at [info@airt.ai](mailto:info@airt.ai).

    After successful verification, you will receive an email with the username and password for the developer account.

    Once you have the credentials, use them to get an access token by calling `get_token` method. It is necessary to get
    an access token; otherwise, you won't be able to access all of the airt service's APIs. You can either pass the username, password, and server
    address as parameters to the `get_token` method or store them in the environment variables **AIRT_SERVICE_USERNAME**,
    **AIRT_SERVICE_PASSWORD**, and **AIRT_SERVER_URL**.

    In addition to the regular authentication with credentials, you can also enable multi-factor authentication (MFA) and single sign-on (SSO)
    for generating tokens.

    To help protect your account, we recommend that you enable multi-factor authentication (MFA). MFA provides additional security by requiring
    you to provide unique verification code (OTP) in addition to your regular sign-in credentials when performing critical operations.

    Your account can be configured for MFA in just two easy steps:

    1. To begin, you need to enable MFA for your account by calling the `User.enable_mfa` method, which will generate a QR code. You can then
    scan the QR code with an authenticator app, such as Google Authenticator and follow the on-device instructions to finish the setup in your smartphone.

    2. Finally, activate MFA for your account by calling `User.activate_mfa` and passing the dynamically generated six-digit verification code from your
    smartphone's authenticator app.

    After activating MFA for your account, you must pass the dynamically generated six-digit verification code, along with your username and password,
    to the `get_token` method to generate new tokens.

    Single sign-on (SSO) can be enabled for your account in three simple steps:

    1. Enable the SSO for a provider by calling the `User.enable_sso` method with the SSO provider name and an email address. At the moment,
    we only support **"google"** and **"github"** as SSO providers. We intend to support additional SSO providers in future releases.

    2. Before you can start generating new tokens with SSO, you must first authenticate with the SSO provider. Call the `get_token` with
    the same SSO provider you have enabled in the step above to generate an SSO authorization URL. Please copy and paste it into your
    preferred browser and complete the authentication process with the SSO provider.

    3. After successfully authenticating with the SSO provider, call the `set_sso_token` method to generate a new token and use it automatically
    in all future interactions with the airt server.

    Here's an example of how to use the Client class to authenticate and display the details of the currently logged-in user.

    Example:
        ```python
        # Importing necessary libraries
        from  airt.client import Client, User

        # Authenticate
        # MFA enabled users must pass the OTP along with the username and password
        # to the get_token method.
        Client.get_token(username="{fill in username}", password="{fill in password}")

        # Print the logged-in user details
        print(User.details())
        ```
    """

    server: Optional[str] = None
    auth_token: Optional[str] = None
    sso_authorization_url: Optional[str] = None

    def __init__(
        self,
        server: str,
        auth_token: str,
        sso_authorization_url: Optional[str] = None,
    ):
        Client.server = server
        Client.auth_token = auth_token
        Client.sso_authorization_url = sso_authorization_url

    @classmethod
    def get_token(  # type: ignore
        cls,
        *,
        username: Optional[str] = None,
        password: Optional[str] = None,
        server: Optional[str] = None,
        sso_provider: Optional[str] = None,
        otp: Optional[str] = None,
    ) -> Optional[str]:
        """Get application token for airt service from a username/password pair.

        This methods validates the developer credentials and returns an auth token. The returned auth
        token is implicitly used in all the interactions with the server.

        If you've already enabled multi-factor authentication (MFA) for your account, you'll need to
        pass the dynamically generated six-digit verification code along with your username and
        password to generate new tokens.

        If the token is requested using Single sign-on (SSO), an authorization URL will be returned.
        Please copy and paste it into your preferred browser and complete the SSO provider
        authentication within 10 minutes. Otherwise, the SSO login will time out and you will need
        to re-request the token.

        Args:
            username: Username for the developer account. If None (default value), then the value from
                **AIRT_SERVICE_USERNAME** environment variable is used.
            password: Password for the developer account. If None (default value), then the value from
                **AIRT_SERVICE_PASSWORD** environment variable is used.
            server: The airt server uri. If None (default value), then the value from **AIRT_SERVER_URL** environment variable
                is used. If the variable is not set as well, then the default public server will be used. Please leave this
                setting to default unless you are running the service in your own server (please email us to info@airt.ai
                for that possibility).
            sso_provider: Name of the Single sign-on (SSO) provider. Please pass this parameter only if you have successfully
                enabled SSO for this provider. At present, the API only supports "google" and "github" as valid SSO providers.
            otp: Dynamically generated six-digit verification code from the authenticator app or the OTP you have received via SMS.

        Returns:
            The authorization url if the token is requested using Single sign-on (SSO).

        Raises:
            ValueError: If the username/password pair does not match.
            ConnectionError: If the server address is invalid or not reachable.
            KeyError: If username/password is neither passed as parameters nor stored in environment variables.

        Here's an example of a non-MFA user authenticating and generating a new token

        Example:
            ```python
            # Importing necessary libraries
            from  airt.client import User, Client

            # Authenticate
            Client.get_token(username="{fill in username}", password="{fill in password}")

            # Print the logged-in user details
            print(User.details())
            ```

        Here's an example of a MFA user authenticating using SMS OTP and generating a new token

        Example:
            ```python
            # Importing necessary libraries
            from  airt.client import Client, User

            # Request OTP via SMS to authenticate
            # If you want to use the OTP from the authenticator app, skip this step and
            # don't generate an SMS OTP; instead, pass the OTP from the authenticator
            # app to the get_token method below
            username="{fill in username}"
            User.send_sms_otp(
                username=username,
                message_template_name="get_token" # Don't change the message_template_name
            )

            # Authenticate using SMS OTP
            # The send_sms_otp method will send the OTP via SMS to the registered
            # phone number, which you must fill below
            password="{fill in password}"
            otp="{fill in otp}"
            Client.get_token(username=username, password=password, otp=otp)

            # Print the logged-in user details
            print(User.details())
            ```
        """
        cls.server = get_base_url(server)

        username, password = _get_credentials(username, password)

        if otp is not None:
            password = json.dumps({"password": password, "user_otp": otp})

        if sso_provider is None:
            response = post_data(
                url=f"{cls.server}/token",
                data=dict(username=username, password=password),
                token=None,
            )

            cls.auth_token = response["access_token"]
        else:
            response = post_data(
                url=f"{cls.server}/sso/initiate",
                data=json.dumps(  # type: ignore
                    dict(
                        username=username, password=password, sso_provider=sso_provider
                    )
                ),
                token=None,
            )

            cls.sso_authorization_url = response["authorization_url"]
            return cls.sso_authorization_url

    @classmethod
    def set_sso_token(cls):
        """Set the application token generated using Single sign-on (SSO).

        The token set using this method will be implicitly used in all the interactions with the server.

        Please call this method only if you successfully enabled and completed the login with the Single
        sign-on (SSO) provider. If not, please call the `get_token` method with an appropriate
        sso_provider to initiate the SSO authentication.

        Here's an example of authenticating with Single sign-on (SSO) using google and setting the
        newly generated token to interact with the airt service.

        Example:
            ```python
            # Importing necessary libraries
            from  airt.client import Client, User

            # Authenticate
            Client.get_token(username="{fill in username}", password="{fill in password}")

            # Enable single sign-on (SSO) and use google as the provider
            sso_provider="google"
            sso_email="{fill in sso_email}"
            User.enable_sso(sso_provider=sso_provider, sso_email=sso_email)

            # Authenticate using Single sign-on (SSO)
            # To generate a token using SSO, you must first authenticate with the provider.
            # The command below will generate an authorization URL for you.
            # Please copy and paste it into your preferred browser and complete the
            # SSO provider authentication within 10 minutes. Otherwise, the SSO login
            # will time out and you will need to call the get_token method again.
            sso_url = Client.get_token(sso_provider=sso_provider)
            print(sso_url)

            # Once the provider authentication is successful, call the below method to
            # set the generated token
            Client.set_sso_token()

            # If set_sso_token fails, the line below will throw an error.
            print(User.details())
            ```
        """
        quoted_authorization_url = urllib.parse.quote(cls.sso_authorization_url)
        response = get_data(
            url=f"{cls.server}/sso/token/?authorization_url={quoted_authorization_url}",
            token=None,
        )

        cls.auth_token = response["access_token"]

    @classmethod
    def set_token(cls, token: Optional[str] = None, server: Optional[str] = None):
        """Set application token for airt service.

        If you already have a valid token, you can call this method to set it and use it in all
        subsequent interactions with the airt server.

        Please call this method only if you already have a token. If not, please call the `get_token` method to generate one.

        Args:
            token: The application token obtained by calling the `get_token` method, or an APIKey obtained by calling
                the `APIKey.create` method. If None (default value), then the value from **AIRT_SERVICE_TOKEN** environment variable is used.
            server: The airt server uri. If None (default value), then the value from **AIRT_SERVER_URL** environment variable
                is used. If the variable is not set as well, then the default public server will be used. Please leave this
                setting to default unless you are running the service in your own server (please email us to info@airt.ai
                for that possibility).

        An example to set an existing token:

        Example:
            ```python
            # Importing necessary libraries
            from  airt.client import Client, User

            # Optional Step: For demonstration purpose, generate a new token
            # When you generate a new token with the get_token method, you do not
            # need to explicitly call set_token. It is shown here for demo purposes only.
            # Skip this step if you already have a valid token and pass it directly to
            # the set_token method below
            Client.get_token(username="{fill in username}", password="{fill in password}")

            # Setting a valid token
            Client.set_token(token=Client.auth_token)

            # If set_token fails, the line below will throw an error.
            print(User.details())
            ```
        """

        auth_token = token if token is not None else os.environ.get(SERVICE_TOKEN)

        if not auth_token:
            raise KeyError(
                f"The token is neither passed as parameter nor set in the environment variable {SERVICE_TOKEN}."
            )

        cls.auth_token = auth_token
        cls.server = get_base_url(server)

    @staticmethod
    def version() -> dict:
        """Return the client and server versions.

        Returns:
            A dict containing the client and server versions.

        Raises:
            ConnectionError: If the server address is invalid or not reachable.

        An example to get the client and server versions:

        Example:
            ```python
            # Importing necessary libraries
            from  airt.client import Client

            # Get the client and server versions
            print(Client.version())
            ```
        """

        response = Client._get_data(relative_url=f"/version")

        version = {
            # nosemgrep: python.lang.security.audit.non-literal-import.non-literal-import
            "client": importlib.import_module(CLIENT_NAME).__version__,
            "server": response["airt_service"],
        }

        return version

    @classmethod
    def _get_server_url_and_token(cls) -> Tuple[Optional[str], Optional[str]]:
        """Fetch the server URL and the auth token.

        Returns:
            A tuple containing server URL and auth token.
        """

        cls.server = get_base_url(cls.server)

        return cls.server, cls.auth_token

    @classmethod
    def _post_data(
        cls,
        relative_url: str,
        data: Optional[Dict[str, Any]] = None,
        json: Optional[Dict[str, Any]] = None,
    ) -> Dict[str, Any]:
        """Make a POST request.

        This method will implicitly add the server base URL and the token for every request.

        Args:
            relative_url: The relative URL of the server's API endpoint.
            data: A Dictionary object to send in the body of the POST request. The data sent in this param will automatically be form-encoded by the request library.
            json: A Dictionary object to send in the body of the POST request. The data sent in this param will automatically be JSON-encoded by the request library.

        Returns:
            Response body as a dictionary.

        Raises:
            ConnectionError: If the server is not reachable.
            ValueError: If the response code is not in range of 200 - 399.
        """

        server, auth_token = Client._get_server_url_and_token()

        return post_data(
            url=f"{server}{relative_url}",
            data=data,
            json=json,
            token=auth_token,
        )

    @classmethod
    def _get_data(cls, relative_url: str) -> Any:
        """Make a GET request.

        This method will implicitly add the server base URL and the token for every request.

        Args:
            relative_url: The relative URL of the API endpoint.

        Returns:
            A dictionary that encapsulates the response body.

        Raises:
            ConnectionError: If the server is not reachable.
            ValueError: If the response code is not in range of 200 - 399.
        """

        server, auth_token = Client._get_server_url_and_token()

        return get_data(url=f"{server}{relative_url}", token=auth_token)

    @classmethod
    def _delete_data(cls, relative_url: str) -> Dict[str, Any]:
        """Make a DELETE request.

        This method will implicitly add the server base URL and the token for every request.

        Args:
            relative_url: The relative URL of the API endpoint.

        Returns:
            A dictionary that encapsulates the response body.

        Raises:
            ConnectionError: If the server is not reachable.
            ValueError: If the response code is not in range of 200 - 399.
        """

        server, auth_token = Client._get_server_url_and_token()

        return delete_data(url=f"{server}{relative_url}", token=auth_token)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../notebooks/API_Helper.ipynb.

# %% auto 0
__all__ = ['ensure_is_instance', 'get_base_url', 'post_data', 'get_data', 'delete_data', 'add_ready_column', 'generate_df',
           'get_values_from_item', 'get_attributes_from_instances', 'dict_to_df', 'standardize_phone_number']

# %% ../notebooks/API_Helper.ipynb 2
from typing import *

# %% ../notebooks/API_Helper.ipynb 3
import os
import requests
from types import MethodType
import textwrap

import pandas as pd

from airt.constant import SERVER_URL, PROD_URL

# %% ../notebooks/API_Helper.ipynb 5
def ensure_is_instance(o: Any, cls: Type):
    """A function to check if the object argument is an instance of the class argument.

    Args:
        o: A python object for which the instance needs to be checked.
        cls: The expected instance of the object argument.

    Raises:
        A TypeError if the object is not an instance of the class type.
    """
    if not isinstance(o, cls):
        raise TypeError(
            f"The parameter must be a {cls} type, but got `{type(o).__name__}`"
        )

# %% ../notebooks/API_Helper.ipynb 7
def _get_json(response: requests.Response) -> Any:
    """A function to validate the status of the response object.

    Args:
        response: The response object that encapsulates the server's response.

    Returns:
        A dictionary of the response body.

    Raises:
        ValueError: If the response code is not in range of 200 - 399.
    """
    if response:
        return response.json()
    else:
        if (
            response.headers.get("content-type") == "application/json"
            and "detail" in response.json()
        ):
            raise ValueError(response.json()["detail"])
        else:
            raise ValueError(response.text)

# %% ../notebooks/API_Helper.ipynb 11
def get_base_url(server: Optional[str]) -> str:
    """Return the base URL for the airt server.

    If the server value is `None`, retrive the value from the environment variable `AIRT_SERVER_URL`.
    If the variable is not set as well, then the default public server will be used.
    """

    return server if server is not None else os.environ.get(SERVER_URL, PROD_URL)

# %% ../notebooks/API_Helper.ipynb 15
def post_data(
    url: str,
    token: Optional[str],
    data: Optional[Dict[str, Any]] = None,
    json: Optional[Dict[str, Any]] = None,
) -> Dict[str, Any]:
    """A function to send a POST request.

    Args:
        url: The URL of the server to which the request needs to be sent.
        data: A Dictionary object to send in the body of the POST request. The data sent in this param will automatically be form-encoded by the request library.
        json: A Dictionary object to send in the body of the POST request. The data sent in this param will automatically be JSON-encoded by the request library.
        token: The unique auth token for the client, obtained via calling the `Client.get_token()` method.
            Set it to `None` in `Client.get_token()` to obtain the token.

    Returns:
        A dictionary that encapsulates the response body.

    Raises:
        ConnectionError: If the server is not reachable.
        ValueError: If the response code is not in range of 200 - 399.
    """
    if token is not None:
        headers = {"Authorization": f"Bearer {token}"}
        response = requests.post(url, json=json, data=data, headers=headers)
    else:
        response = requests.post(url, data=data, json=json)
    return _get_json(response)

# %% ../notebooks/API_Helper.ipynb 18
def get_data(url: str, token: Optional[str]) -> Any:
    """Send a GET request.

    Args:
        url: The URL of the server to which the request needs to be sent.
        token: The unique auth token for the client, obtained via calling the `Client.get_token()` method.

    Returns:
        A dictionary that encapsulates the response body.

    Raises:
        ConnectionError: If the server is not reachable.
        ValueError: If the response code is not in range of 200 - 399.
    """
    headers = {"Authorization": f"Bearer {token}"}
    response = requests.get(url, headers=headers)
    return _get_json(response)

# %% ../notebooks/API_Helper.ipynb 20
def delete_data(url: str, token: Optional[str]) -> Dict[str, Any]:
    """Send a DELETE request.

    Args:
        url: The URL of the server to which the request needs to be sent.
        token: The unique auth token for the client, obtained via calling the `Client.get_token()` method.

    Returns:
        A dictionary that encapsulates the response body.

    Raises:
        ConnectionError: If the server is not reachable.
        ValueError: If the response code is not in range of 200 - 399.
    """
    headers = {"Authorization": f"Bearer {token}"}
    response = requests.delete(url, headers=headers)
    return _get_json(response)

# %% ../notebooks/API_Helper.ipynb 23
def add_ready_column(df: pd.DataFrame) -> pd.DataFrame:
    """Add ready column to the DataFrame

    Args:
        df: A pandas DataFrame with completed_steps and total_steps columns

    Returns:
        A pandas DataFrame with ready column
    """

    df["ready"] = df["completed_steps"] == df["total_steps"]
    return df.drop(columns=["completed_steps", "total_steps"])

# %% ../notebooks/API_Helper.ipynb 25
def generate_df(
    items: Union[Dict[str, Any], List[Dict[str, Any]]], columns: list
) -> pd.DataFrame:
    """Generate a DataFrame based on the items length

    Args:
        items: A list encapsulating the response from an API endpoint that needs to be converted into a DataFrame.
        columns: A list of columns names to be included in the DataFrame.

    Returns:
        A DataFrame with a shape of (items, columns), if the length of the items is > 0, otherwise an empty DataFrame with only columns names.
    """

    if len(items) > 0:
        df = pd.DataFrame(items)[columns]
    else:
        df = pd.DataFrame({c: [] for c in columns})

    return df

# %% ../notebooks/API_Helper.ipynb 27
def get_values_from_item(items: list, value: Optional[str] = None) -> str:
    """Get **values** from items seperated by comma.

    Args:
        items: The item list from the response.
        value: The value to extract from each items.

    Returns:
        The **values** as string seperated by comma. If the tags list is empty, <none> will be returned.
    """

    if len(items) == 0:
        return "<none>"

    if value is None:
        return ", ".join(items)

    return ", ".join([str(i[value]) for i in items])

# %% ../notebooks/API_Helper.ipynb 31
def get_attributes_from_instances(
    ox: List[object], attributes: List[str]
) -> List[Dict[str, Any]]:
    """Extract the **attributes** from the instances.

    Args:
        ox: List of instances.
        attributes: Attributes to extract from the instances as a list

    Returns:
        A list encapsulating the attribute name and value pairs of each instance.
    """

    lists = [{i: getattr(o, i) for i in attributes} for o in ox]
    return lists

# %% ../notebooks/API_Helper.ipynb 33
def dict_to_df(d: Dict[str, Any]) -> pd.DataFrame:
    """Convert the dict into a pandas dataframe

    Args:
        d: Dict containing the data and dtypes

    Returns:
        The pandas dataframe constructed from the dict
    """
    data = d["data"]
    dtypes = d["dtypes"]

    df = pd.DataFrame(
        data=data["data"], index=data["index"], columns=data["columns"]
    ).rename_axis(data["index_names"])

    for k, v in dtypes.items():
        df[k] = df[k].astype(v)

    return df

# %% ../notebooks/API_Helper.ipynb 35
def check_and_append_otp_query_param(relative_url: str, otp: Union[str, None]) -> str:
    """Append the otp query parameter to the relative url if its not None

    Args:
        relative_url: The relative url for the route
        otp: user otp

    Returns:
        The updated relative_url if otp is not None, else returns the original relative_url
    """
    if otp is not None:
        relative_url = (
            (relative_url + f"&otp={otp}")
            if len(relative_url.split("?")) > 1
            else (relative_url + f"?otp={otp}")
        )
    return relative_url

# %% ../notebooks/API_Helper.ipynb 39
def standardize_phone_number(phone_number: str) -> str:
    """Standardize the user's phone number

    This function takes the user's phone number in different formats and converts it into a
    standardized format. For example, the user can enter the phone number as **440123456789, +440123456789,
    00440123456789, +44 0123456789, and (+44) 012 345 6789** to register a UK-based phone number and this function takes the user input and
    converts it into 440123456789 standardized format.

    Args:
        phone_number: The phone number to convert into a standardized format.

    Returns:
        The phone number in a standardized format.
    """
    phone_number = "".join(filter(str.isdigit, phone_number))
    if phone_number.startswith("00"):
        phone_number = phone_number[2:]
    return phone_number

# %% ../notebooks/API_Helper.ipynb 41
def add_example_to_docs(o: Any, example: str):
    """Add the given example to the object

    Args:
        o: an object, typically a function or a class, for which the example needs to be added
        example: The example string to add
    """

    original_doc = o.__doc__

    first_line, everything_else = original_doc.split("\n", 1)
    doc_with_example_added = (
        first_line
        + "\n"
        + textwrap.dedent(everything_else)
        + "\n"
        + textwrap.dedent(example)
    )

    o.__doc__ = doc_with_example_added



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../notebooks/CLI_DataBlob.ipynb.

# %% auto 0
__all__ = ['logger']

# %% ../../notebooks/CLI_DataBlob.ipynb 3
from typing import *

# %% ../../notebooks/CLI_DataBlob.ipynb 4
import os
import json

import typer
from typer import echo
from tabulate import tabulate
import datetime as dt
import pandas as pd

from airt.client import Client
from airt.cli import helper
from airt.logger import get_logger, set_level
from airt.constant import CLIENT_DB_USERNAME, CLIENT_DB_PASSWORD

# %% ../../notebooks/CLI_DataBlob.ipynb 6
app = typer.Typer(
    help="""A set of commands for importing and processing data from sources such as CSV/parquet files, databases, AWS S3 buckets, and Azure Blob Storage."""
)

# %% ../../notebooks/CLI_DataBlob.ipynb 8
logger = get_logger(__name__)

# %% ../../notebooks/CLI_DataBlob.ipynb 18
@app.command()
@helper.display_formated_table
@helper.requires_auth_token
def details(
    uuid: str = typer.Argument(
        ...,
        help="Datablob uuid.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Return details of a datablob."""

    from airt.client import DataBlob

    db = DataBlob(uuid=uuid)
    df = db.details()

    df["pulled_on"] = helper.humanize_date(df["pulled_on"])
    df["folder_size"] = helper.humanize_size(df["folder_size"])

    return {"df": df}

# %% ../../notebooks/CLI_DataBlob.ipynb 22
@app.command("from-s3")
@helper.requires_auth_token
def from_s3(
    uri: str = typer.Argument(..., help="The AWS S3 bucket uri."),
    access_key: Optional[str] = typer.Option(
        None,
        help="Access key for the S3 bucket. If **None** (default value), then the value from **AWS_ACCESS_KEY_ID** environment variable is used.",
    ),
    secret_key: Optional[str] = typer.Option(
        None,
        help="Secret key for the S3 bucket. If **None** (default value), then the value from **AWS_SECRET_ACCESS_KEY** environment variable is used.",
    ),
    cloud_provider: Optional[str] = typer.Option(
        None,
        "--cloud-provider",
        "-cp",
        help="The destination cloud storage provider's name to store the datablob. Currently, the API only supports **aws** and **azure** as cloud storage providers. If **None** (default value), then **aws**  will be used as the cloud storage provider.",
    ),
    region: Optional[str] = typer.Option(
        None,
        "--region",
        "-r",
        help="The destination cloud provider's region to save your datablob. If **None** (default value) then the default region will be assigned based on the cloud provider. "
        "In the case of **aws**, the datablob's source bucket region will be used and in the case of **azure**, **westeurope** will be used. "
        "The supported AWS regions are: ap-northeast-1, ap-northeast-2, ap-south-1, ap-southeast-1, ap-southeast-2, ca-central-1, eu-central-1, "
        "eu-north-1, eu-west-1, eu-west-2, eu-west-3, sa-east-1, us-east-1, us-east-2, us-west-1, us-west-2. The supported Azure Blob Storage "
        "regions are: australiacentral, australiacentral2, australiaeast, australiasoutheast, brazilsouth, canadacentral, canadaeast, centralindia, "
        "centralus, eastasia, eastus, eastus2, francecentral, francesouth, germanynorth, germanywestcentral, japaneast, japanwest, koreacentral, koreasouth, "
        "northcentralus, northeurope, norwayeast, norwaywest, southafricanorth, southafricawest, southcentralus, southeastasia, southindia, switzerlandnorth, "
        "switzerlandwest, uaecentral, uaenorth, uksouth, ukwest, westcentralus, westeurope, westindia, westus, westus2.",
    ),
    tag: Optional[str] = typer.Option(
        None,
        "--tag",
        "-t",
        help="A string to tag the datablob. If not passed, then the tag **latest** will be assigned to the datablob.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output datablob uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Create and return a datablob that encapsulates the data from an AWS S3 bucket."""

    from airt.client import DataBlob

    db = DataBlob.from_s3(
        uri=uri,
        access_key=access_key,
        secret_key=secret_key,
        cloud_provider=cloud_provider,
        region=region,
        tag=tag,
    )

    if quiet:
        db.wait()

        typer.echo(f"{db.uuid}")
    else:
        typer.echo(f"Pulling datablob uuid: {db.uuid}")

        db.progress_bar()

# %% ../../notebooks/CLI_DataBlob.ipynb 26
@app.command("from-azure-blob-storage")
@helper.requires_auth_token
def from_azure_blob_storage(
    uri: str = typer.Argument(..., help="Azure Blob Storage URI of the source file."),
    credential: str = typer.Option(
        ...,
        "--credential",
        "-c",
        help="Credential to access the Azure Blob Storage.",
    ),
    cloud_provider: Optional[str] = typer.Option(
        None,
        "--cloud-provider",
        "-cp",
        help="The destination cloud storage provider's name to store the datablob. Currently, the API only supports **aws** and **azure** as cloud storage providers. If **None** (default value), then **azure**  will be used as the cloud storage provider.",
    ),
    region: Optional[str] = typer.Option(
        None,
        "--region",
        "-r",
        help="The destination cloud provider's region to save your datablob. If **None** (default value) then the default region will be assigned based on the cloud provider. "
        "In the case of **aws**, **eu-west-1** will be used and in the case of **azure**, **westeurope** will be used. "
        "The supported AWS regions are: ap-northeast-1, ap-northeast-2, ap-south-1, ap-southeast-1, ap-southeast-2, ca-central-1, eu-central-1, "
        "eu-north-1, eu-west-1, eu-west-2, eu-west-3, sa-east-1, us-east-1, us-east-2, us-west-1, us-west-2. The supported Azure Blob Storage "
        "regions are: australiacentral, australiacentral2, australiaeast, australiasoutheast, brazilsouth, canadacentral, canadaeast, centralindia, "
        "centralus, eastasia, eastus, eastus2, francecentral, francesouth, germanynorth, germanywestcentral, japaneast, japanwest, koreacentral, koreasouth, "
        "northcentralus, northeurope, norwayeast, norwaywest, southafricanorth, southafricawest, southcentralus, southeastasia, southindia, switzerlandnorth, "
        "switzerlandwest, uaecentral, uaenorth, uksouth, ukwest, westcentralus, westeurope, westindia, westus, westus2.",
    ),
    tag: Optional[str] = typer.Option(
        None,
        "--tag",
        "-t",
        help="A string to tag the datablob. If not passed, then the tag **latest** will be assigned to the datablob.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output datablob uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Create and return a datablob that encapsulates the data from an Azure Blob Storage."""

    from airt.client import DataBlob

    db = DataBlob.from_azure_blob_storage(
        uri=uri,
        credential=credential,
        cloud_provider=cloud_provider,
        region=region,
        tag=tag,
    )

    if quiet:
        db.wait()

        typer.echo(f"{db.uuid}")
    else:
        typer.echo(f"Pulling datablob uuid: {db.uuid}")

        db.progress_bar()

# %% ../../notebooks/CLI_DataBlob.ipynb 30
@app.command("from-mysql")
@helper.requires_auth_token
def from_mysql(
    host: str = typer.Option(..., help="Remote database host name."),
    database: str = typer.Option(..., help="Database name."),
    table: str = typer.Option(..., help="Table name."),
    port: int = typer.Option(
        3306,
        help="Host port number. If not passed, then the default value **3306** will be used.",
    ),
    cloud_provider: Optional[str] = typer.Option(
        None,
        "--cloud-provider",
        "-cp",
        help="The destination cloud storage provider's name to store the datablob. Currently, the API only supports **aws** and **azure** as cloud storage providers. If **None** (default value), then **aws**  will be used as the cloud storage provider.",
    ),
    region: Optional[str] = typer.Option(
        None,
        "--region",
        "-r",
        help="The destination cloud provider's region to save your datablob. If **None** (default value) then the default region will be assigned based on the cloud provider. "
        "In the case of **aws**, **eu-west-1** will be used and in the case of **azure**, **westeurope** will be used. "
        "The supported AWS regions are: ap-northeast-1, ap-northeast-2, ap-south-1, ap-southeast-1, ap-southeast-2, ca-central-1, eu-central-1, "
        "eu-north-1, eu-west-1, eu-west-2, eu-west-3, sa-east-1, us-east-1, us-east-2, us-west-1, us-west-2. The supported Azure Blob Storage "
        "regions are: australiacentral, australiacentral2, australiaeast, australiasoutheast, brazilsouth, canadacentral, canadaeast, centralindia, "
        "centralus, eastasia, eastus, eastus2, francecentral, francesouth, germanynorth, germanywestcentral, japaneast, japanwest, koreacentral, koreasouth, "
        "northcentralus, northeurope, norwayeast, norwaywest, southafricanorth, southafricawest, southcentralus, southeastasia, southindia, switzerlandnorth, "
        "switzerlandwest, uaecentral, uaenorth, uksouth, ukwest, westcentralus, westeurope, westindia, westus, westus2.",
    ),
    username: Optional[str] = typer.Option(
        None,
        "--username",
        "-u",
        help=f'Database username. If not passed, the default value "root" will be used unless the value is explicitly set in the environment variable **{CLIENT_DB_USERNAME}**.',
    ),
    password: Optional[str] = typer.Option(
        None,
        "--password",
        "-p",
        help=f'Database password. If not passed, the default value "" will be used unless the value is explicitly set in the environment variable **{CLIENT_DB_PASSWORD}**.',
    ),
    tag: Optional[str] = typer.Option(
        None,
        "--tag",
        "-t",
        help="A string to tag the datablob. If not passed, then the tag **latest** will be assigned to the datablob.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output datablob uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Create and return a datablob that encapsulates the data from a mysql database.

    If the database requires authentication, pass the username/password as commandline arguments or store it in
    the **AIRT_CLIENT_DB_USERNAME** and **AIRT_CLIENT_DB_PASSWORD** environment variables.
    """

    from airt.client import DataBlob

    db = DataBlob.from_mysql(
        host=host,
        database=database,
        port=port,
        table=table,
        username=username,
        password=password,
        cloud_provider=cloud_provider,
        region=region,
        tag=tag,
    )

    if quiet:
        db.wait()
        typer.echo(f"{db.uuid}")
    else:
        typer.echo(f"Pulling datablob uuid: {db.uuid}")
        db.progress_bar()

# %% ../../notebooks/CLI_DataBlob.ipynb 34
@app.command("to-datasource")
@helper.requires_auth_token
def to_datasource(
    uuid: str = typer.Option(
        ...,
        help="Datablob uuid.",
    ),
    file_type: str = typer.Option(
        ...,
        help='The file type of the datablob. Currently, the API only supports "csv" and "parquet" as file types.',
    ),
    index_column: str = typer.Option(
        ...,
        help="The column to use as index (row labels).",
    ),
    sort_by: str = typer.Option(
        ...,
        help="The column(s) to sort the data. Can either be a string or a JSON encoded list of strings.",
    ),
    deduplicate_data: bool = typer.Option(
        False,
        help="If set to **True** (default value **False**), the datasource will be created with duplicate rows removed.",
    ),
    blocksize: str = typer.Option(
        "256MB",
        help="The number of bytes used to split larger files. If None, then the default value **256MB** will be used.",
    ),
    kwargs_json: Optional[str] = typer.Option(
        None,
        help="Additional JSON encoded dict arguments to use while processing the data.e.g: To skip 100 lines from the bottom of file, pass '{"
        '"skipfooter"'
        ": 100}'",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output datasource uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Process the datablob and return a datasource object."""

    from airt.client import DataBlob

    kwargs = json.loads(kwargs_json) if kwargs_json else {}

    try:
        sort_by = json.loads(sort_by)

    except json.JSONDecodeError as e:
        pass

    db = DataBlob(uuid=uuid)
    ds = db.to_datasource(
        file_type=file_type,
        index_column=index_column,
        sort_by=sort_by,
        deduplicate_data=deduplicate_data,
        blocksize=blocksize,
        **kwargs,
    )

    if quiet:
        ds.wait()
        typer.echo(f"{ds.uuid}")
    else:
        typer.echo(f"Processing and pulling the datasource uuid: {ds.uuid}")

        ds.progress_bar()

# %% ../../notebooks/CLI_DataBlob.ipynb 38
@app.command()
@helper.display_formated_table
@helper.requires_auth_token
def ls(
    offset: int = typer.Option(
        0,
        "--offset",
        "-o",
        help="The number of datablobs to offset at the beginning. If **None**, then the default value **0** will be used.",
    ),
    limit: int = typer.Option(
        100,
        "--limit",
        "-l",
        help="The maximum number of datablobs to return from the server. If **None**, then the default value **100** will be used.",
    ),
    disabled: bool = typer.Option(
        False,
        "--disabled",
        help="If set to **True**, then only the deleted datablobs will be returned."
        "Else, the default value **False** will be used to return only the list"
        "of active datablobs.",
    ),
    completed: bool = typer.Option(
        False,
        "--completed",
        help="If set to **True**, then only the datablobs that are successfully downloaded"
        "to the server will be returned. Else, the default value **False** will be used to"
        "return all the datablobs.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output only datablob uuids separated by space",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Return the list of datablobs."""

    from airt.client import DataBlob

    dbx = DataBlob.ls(
        offset=offset, limit=limit, disabled=disabled, completed=completed
    )

    df = DataBlob.as_df(dbx)

    df["pulled_on"] = helper.humanize_date(df["pulled_on"])
    df["folder_size"] = helper.humanize_size(df["folder_size"])

    return {"df": df, "quite_column_name": "datablob_uuid"}

# %% ../../notebooks/CLI_DataBlob.ipynb 43
@app.command()
@helper.display_formated_table
@helper.requires_auth_token
def rm(
    uuid: str = typer.Argument(
        ...,
        help="Datablob uuid.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output the deleted datablob uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Delete a datablob from the server."""

    from airt.client import DataBlob

    db = DataBlob(uuid=uuid)
    df = db.delete()

    df["pulled_on"] = helper.humanize_date(df["pulled_on"])
    df["folder_size"] = helper.humanize_size(df["folder_size"])

    return {"df": df, "quite_column_name": "datablob_uuid"}

# %% ../../notebooks/CLI_DataBlob.ipynb 47
@app.command()
@helper.display_formated_table
@helper.requires_auth_token
def tag(
    uuid: str = typer.Option(
        ...,
        "--datablob_uuid",
        "-uuid",
        help="Datablob uuid in the server.",
    ),
    name: str = typer.Option(
        ...,
        "--name",
        "-n",
        help="A string to tag the datablob.",
    ),
    format: Optional[str] = typer.Option(
        None,
        "--format",
        "-f",
        help="Format output and show only the given column(s) values.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> Dict["str", Union[pd.DataFrame, str]]:
    """Tag an existing datablob in the server."""

    from airt.client import DataBlob

    db = DataBlob(uuid=uuid)
    df = db.tag(name=name)

    df["pulled_on"] = helper.humanize_date(df["pulled_on"])
    df["folder_size"] = helper.humanize_size(df["folder_size"])

    return {"df": df}

# %% ../../notebooks/CLI_DataBlob.ipynb 50
@app.command("from-local")
@helper.requires_auth_token
def from_local(
    path: str = typer.Option(
        ...,
        "--path",
        "-p",
        help="The relative or absolute path to a local CSV/parquet file or to a directory containing the CSV/parquet files.",
    ),
    cloud_provider: Optional[str] = typer.Option(
        None,
        "--cloud-provider",
        "-cp",
        help="The destination cloud storage provider's name to store the datablob. Currently, the API only supports **aws** and **azure** as cloud storage providers. If **None** (default value), then **aws**  will be used as the cloud storage provider.",
    ),
    region: Optional[str] = typer.Option(
        None,
        "--region",
        "-r",
        help="The destination cloud provider's region to save your datablob. If **None** (default value) then the default region will be assigned based on the cloud provider. "
        "In the case of **aws**, **eu-west-1** will be used and in the case of **azure**, **westeurope** will be used. "
        "The supported AWS regions are: ap-northeast-1, ap-northeast-2, ap-south-1, ap-southeast-1, ap-southeast-2, ca-central-1, eu-central-1, "
        "eu-north-1, eu-west-1, eu-west-2, eu-west-3, sa-east-1, us-east-1, us-east-2, us-west-1, us-west-2. The supported Azure Blob Storage "
        "regions are: australiacentral, australiacentral2, australiaeast, australiasoutheast, brazilsouth, canadacentral, canadaeast, centralindia, "
        "centralus, eastasia, eastus, eastus2, francecentral, francesouth, germanynorth, germanywestcentral, japaneast, japanwest, koreacentral, koreasouth, "
        "northcentralus, northeurope, norwayeast, norwaywest, southafricanorth, southafricawest, southcentralus, southeastasia, southindia, switzerlandnorth, "
        "switzerlandwest, uaecentral, uaenorth, uksouth, ukwest, westcentralus, westeurope, westindia, westus, westus2.",
    ),
    tag: Optional[str] = typer.Option(
        None,
        "--tag",
        "-t",
        help="A string to tag the datablob. If not passed, then the tag **latest** will be assigned to the datablob.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output data id only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
) -> None:
    """Create and return a datablob from local csv file.

    The API currently allows users to create datablobs from CSV or Parquet files. We intend to support additional file formats in future releases.
    """

    from airt.client import DataBlob

    if quiet:
        db = DataBlob.from_local(
            path=path,
            cloud_provider=cloud_provider,
            region=region,
            tag=tag,
            show_progress=False,
        )
        typer.echo(f"{db.uuid}")
    else:
        db = DataBlob.from_local(
            path=path, cloud_provider=cloud_provider, region=region, tag=tag
        )
        typer.echo(f"Successfully pulled the datablob uuid: {db.uuid}.")

# %% ../../notebooks/CLI_DataBlob.ipynb 54
@app.command("from-clickhouse")
@helper.requires_auth_token
def from_clickhouse(
    host: str = typer.Option(..., help="Remote database host name."),
    database: str = typer.Option(..., help="Database name."),
    table: str = typer.Option(..., help="Table name."),
    protocol: str = typer.Option(
        ..., help='Protocol to use. The valid values are "native" and "http".'
    ),
    index_column: str = typer.Option(
        ..., help="The column to use as index (row labels)."
    ),
    timestamp_column: str = typer.Option(
        ..., help="Timestamp column name in the tabel."
    ),
    port: int = typer.Option(
        0,
        help="Host port number. If not passed, then the default value **0** will be used.",
    ),
    cloud_provider: Optional[str] = typer.Option(
        None,
        "--cloud-provider",
        "-cp",
        help="The destination cloud storage provider's name to store the datablob. Currently, the API only supports **aws** and **azure** as cloud storage providers. If **None** (default value), then **aws**  will be used as the cloud storage provider.",
    ),
    region: Optional[str] = typer.Option(
        None,
        "--region",
        "-r",
        help="The destination cloud provider's region to save your datablob. If **None** (default value) then the default region will be assigned based on the cloud provider. "
        "In the case of **aws**, **eu-west-1** will be used and in the case of **azure**, **westeurope** will be used. "
        "The supported AWS regions are: ap-northeast-1, ap-northeast-2, ap-south-1, ap-southeast-1, ap-southeast-2, ca-central-1, eu-central-1, "
        "eu-north-1, eu-west-1, eu-west-2, eu-west-3, sa-east-1, us-east-1, us-east-2, us-west-1, us-west-2. The supported Azure Blob Storage "
        "regions are: australiacentral, australiacentral2, australiaeast, australiasoutheast, brazilsouth, canadacentral, canadaeast, centralindia, "
        "centralus, eastasia, eastus, eastus2, francecentral, francesouth, germanynorth, germanywestcentral, japaneast, japanwest, koreacentral, koreasouth, "
        "northcentralus, northeurope, norwayeast, norwaywest, southafricanorth, southafricawest, southcentralus, southeastasia, southindia, switzerlandnorth, "
        "switzerlandwest, uaecentral, uaenorth, uksouth, ukwest, westcentralus, westeurope, westindia, westus, westus2.",
    ),
    username: Optional[str] = typer.Option(
        None,
        "--username",
        "-u",
        help="Database username. If not passed, the default value 'root' will be used unless the value is explicitly set in the environment variable **CLICKHOUSE_USERNAME**.",
    ),
    password: Optional[str] = typer.Option(
        None,
        "--password",
        "-p",
        help="Database password. If not passed, the default value '' will be used unless the value is explicitly set in the environment variable **CLICKHOUSE_PASSWORD**.",
    ),
    filters_json: Optional[str] = typer.Option(
        None,
        "--filters-json",
        "-f",
        help="Additional parameters to be used when importing data. For example, if you want to filter and extract data only for a specific user_id, pass '{"
        '"user_id"'
        ": 1}'.",
    ),
    tag: Optional[str] = typer.Option(
        None,
        "--tag",
        "-t",
        help="A string to tag the datablob. If not passed, then the tag **latest** will be assigned to the datablob.",
    ),
    quiet: bool = typer.Option(
        False,
        "--quiet",
        "-q",
        help="Output datablob uuid only.",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        "-d",
        help="Set logger level to DEBUG and output everything.",
    ),
):
    """Create and return a datablob that encapsulates the data from a ClickHouse database.

    If the database requires authentication, pass the username/password as commandline arguments or store it in
    the **CLICKHOUSE_USERNAME** and **CLICKHOUSE_PASSWORD** environment variables.
    """

    filters = json.loads(filters_json) if filters_json else None

    from airt.client import DataBlob

    db = DataBlob.from_clickhouse(
        host=host,
        database=database,
        table=table,
        protocol=protocol,
        index_column=index_column,
        timestamp_column=timestamp_column,
        port=port,
        username=username,
        password=password,
        filters=filters,
        cloud_provider=cloud_provider,
        region=region,
        tag=tag,
    )

    if quiet:
        db.wait()
        typer.echo(f"{db.uuid}")
    else:
        typer.echo(f"Pulling datablob uuid: {db.uuid}")
        db.progress_bar()
